
/*  This file is part of the program psim.

    Copyright (C) 1994-1995, Andrew Cagney <cagney@highland.com.au>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, see <http://www.gnu.org/licenses/>.

    --

    This file was generated by the program igen */

#ifndef _ICACHE_C_
#define _ICACHE_C_


#include "cpu.h"
#include "idecode.h"
#include "semantics.h"
#include "icache.h"
#ifdef HAVE_COMMON_FPU
#include "sim-inline.h"
#include "sim-fpu.h"
#endif
#include "support.h"


PSIM_INLINE_ICACHE\
(unsigned_word)
icache_illegal
(cpu *processor,
 instruction_word instruction,
 unsigned_word cia,
 idecode_cache *cache_entry)
{
  /* semantic routine */
#line 982 "./ppc-instructions"
  program_interrupt(processor, cia,
                    illegal_instruction_program_interrupt);
  error("Internal function must longjump\n");
  return 0;
#line 53 "icache.c"
}

PSIM_INLINE_ICACHE\
(unsigned_word)
icache_floating_point_unavailable
(cpu *processor,
 instruction_word instruction,
 unsigned_word cia,
 idecode_cache *cache_entry)
{
  /* semantic routine */
#line 990 "./ppc-instructions"
  floating_point_unavailable_interrupt(processor, cia);
  error("Internal function must longjump\n");
  return 0;
#line 69 "icache.c"
}


EXTERN_ICACHE\
(unsigned_word)
icache_instruction_call(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_instruction_call
#undef MY_PREFIX
#define MY_PREFIX instruction_call
#line 1493 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode instruction_call\n"));
#line 86 "icache.c"

  /* validate: 0.0,6./,11./,16./,21./,31.1 */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x1)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.0,6./,11./,16./,21./,31.1 #define get-values-from-icache|put-values-in-icache */
#line 95 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_instruction_call;

    /* semantic routine */
#line 1493 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics instruction_call\n"));
#line 105 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_instruction_call, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1493 "./ppc-instructions"
    {
      if (!os_emul_instruction_call(processor, cia, real_addr(cia, 1)))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
    }
#line 120 "icache.c"
    return nia;
    /* extraction: 0.0,6./,11./,16./,21./,31.1 #undef get-values-from-icache|put-values-in-icache */
#line 123 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Trap_Word_Immediate(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Trap_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Trap_Word_Immediate
#line 2897 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Trap Word Immediate\n"));
#line 142 "icache.c"

  /* validate: 0.3,6.TO,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.3,6.TO,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 2897 "./ppc-instructions"
    unsigned const TO UNUSED = cache_entry->crack.D.TO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 162 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Trap_Word_Immediate;

    /* semantic routine */
#line 2897 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Trap Word Immediate\n"));
#line 172 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Trap_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2897 "./ppc-instructions"
    {
      signed_word a = EXTENDED(*rA);
      signed_word b = EXTS(SI);
      if ((a < b && TO_0_)
          || (a > b && TO_1_)
          || (a == b && TO_2_)
          || ((unsigned_word)a < (unsigned_word)b && TO_3_)
          || ((unsigned_word)a > (unsigned_word)b && TO_4_)
          )
        program_interrupt(processor, cia,
                          trap_program_interrupt);
    }
#line 194 "icache.c"
    return nia;
    /* extraction: 0.3,6.TO,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 198 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Immediate(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Immediate
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Immediate
#line 2687 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Immediate\n"));
#line 217 "icache.c"

  /* validate: 0.7,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x1c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.7,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 241 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Immediate;

    /* semantic routine */
#line 2687 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Immediate\n"));
#line 251 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2687 "./ppc-instructions"
    {
      signed_word prod = *rA * EXTS(SI);
      *rT = prod;
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 266 "icache.c"
    return nia;
    /* extraction: 0.7,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 270 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Immediate_Carrying(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Immediate_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Immediate_Carrying
#line 2574 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Immediate Carrying\n"));
#line 289 "icache.c"

  /* validate: 0.8,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x20000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.8,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 313 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Immediate_Carrying;

    /* semantic routine */
#line 2574 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Immediate Carrying\n"));
#line 323 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Immediate_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2574 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(EXTS(SI));
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 0/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 341 "icache.c"
    return nia;
    /* extraction: 0.8,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 345 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Logical_Immediate(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Logical_Immediate
#undef MY_PREFIX
#define MY_PREFIX Compare_Logical_Immediate
#line 2833 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare Logical Immediate\n"));
#line 364 "icache.c"

  /* validate: 0.10,6.BF,9./,10.L,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc400000) != 0x28000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.10,6.BF,9./,10.L,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.D.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.D.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2833 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.D.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2833 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 388 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Logical_Immediate;

    /* semantic routine */
#line 2833 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare Logical Immediate\n"));
#line 398 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Logical_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2833 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        unsigned_word a;
        unsigned_word b = UI;
        if (L == 0)
          a = MASKED(*rA, 32, 63);
        else
          a = *rA;
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK, BF_BITMASK);
    }
#line 423 "icache.c"
    return nia;
    /* extraction: 0.10,6.BF,9./,10.L,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 427 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Immediate(cpu *processor,
                         instruction_word instruction,
                         unsigned_word cia,
                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Immediate
#undef MY_PREFIX
#define MY_PREFIX Compare_Immediate
#line 2791 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare Immediate\n"));
#line 446 "icache.c"

  /* validate: 0.11,6.BF,9./,10.L,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc400000) != 0x2c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.11,6.BF,9./,10.L,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.D.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.D.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2791 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.D.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 470 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Immediate;

    /* semantic routine */
#line 2791 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare Immediate\n"));
#line 480 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2791 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        signed_word a;
        signed_word b = EXTS(SI);
        if (L == 0)
          a = EXTENDED(*rA);
        else
          a = *rA;
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK, BF_BITMASK);
    }
#line 505 "icache.c"
    return nia;
    /* extraction: 0.11,6.BF,9./,10.L,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 509 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Carrying(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Carrying
#line 2554 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Carrying\n"));
#line 528 "icache.c"

  /* validate: 0.12,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x30000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.12,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 552 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Carrying;

    /* semantic routine */
#line 2554 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Carrying\n"));
#line 562 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2554 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(EXTS(SI));
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 0/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 578 "icache.c"
    return nia;
    /* extraction: 0.12,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 582 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Carrying_and_Record(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Carrying_and_Record
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Carrying_and_Record
#line 2564 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Carrying and Record\n"));
#line 601 "icache.c"

  /* validate: 0.13,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x34000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.13,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 625 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Carrying_and_Record;

    /* semantic routine */
#line 2564 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Carrying and Record\n"));
#line 635 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Carrying_and_Record, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2564 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(EXTS(SI));
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 1/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 1/*Rc*/);
    }
#line 651 "icache.c"
    return nia;
    /* extraction: 0.13,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 655 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_RA0(cpu *processor,
                         instruction_word instruction,
                         unsigned_word cia,
                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_RA0
#line 2512 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate\n"));
#line 674 "icache.c"

  /* validate: 0.14,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x38000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = 0; /* in icache - constant field */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 1 /* RA == 0 */ /* semantic compute - constant compare */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 698 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_RA0;

    /* semantic routine */
#line 2512 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate\n"));
#line 708 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2512 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI);
      else		*rT = *rA + EXTS(SI);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 724 "icache.c"
    return nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 728 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_RA(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_RA
#line 2512 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate\n"));
#line 747 "icache.c"

  /* validate: 0.14,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x38000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 0 /* RA == 0 */ /* semantic compute - constant compare */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 771 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_RA;

    /* semantic routine */
#line 2512 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate\n"));
#line 781 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2512 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI);
      else		*rT = *rA + EXTS(SI);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 797 "icache.c"
    return nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 801 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Shifted_RA0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Shifted_RA0
#line 2522 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Shifted\n"));
#line 820 "icache.c"

  /* validate: 0.15,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x3c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = 0; /* in icache - constant field */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 1 /* RA == 0 */ /* semantic compute - constant compare */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 844 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Shifted_RA0;

    /* semantic routine */
#line 2522 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Shifted\n"));
#line 854 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2522 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI) << 16;
      else		*rT = *rA + (EXTS(SI) << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 870 "icache.c"
    return nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 874 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Shifted_RA(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Shifted_RA
#line 2522 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Shifted\n"));
#line 893 "icache.c"

  /* validate: 0.15,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x3c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 0 /* RA == 0 */ /* semantic compute - constant compare */
#line 64 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 917 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Shifted_RA;

    /* semantic routine */
#line 2522 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Shifted\n"));
#line 927 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2522 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI) << 16;
      else		*rT = *rA + (EXTS(SI) << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 943 "icache.c"
    return nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 947 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO0
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 966 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 986 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO0;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 996 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1045 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1049 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO1
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1068 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1088 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO1;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1098 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1147 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1151 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO2
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1170 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1190 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO2;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1200 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1249 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1253 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO3
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1272 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1292 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO3;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1302 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1351 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1355 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO4
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1374 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1394 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO4;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1404 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1453 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1457 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO5
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1476 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1496 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO5;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1506 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1555 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1559 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO6
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1578 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1598 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO6;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1608 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1657 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1661 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO7
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1680 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1700 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO7;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1710 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1759 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1763 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO8
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1782 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1802 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO8;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1812 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1861 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1865 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO9
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1884 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1904 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO9;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1914 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1963 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1967 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO10
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1986 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2006 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO10;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2016 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2065 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2069 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO11
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2088 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2108 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO11;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2118 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2167 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2171 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO12
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2190 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2210 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO12;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2220 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2269 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2273 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO13
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2292 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2312 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO13;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2322 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2371 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2375 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO14
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2394 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2414 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO14;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2424 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2473 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2477 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO15
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2496 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2516 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO15;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2526 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2575 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2579 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO16
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2598 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2618 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO16;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2628 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2677 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2681 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO17
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2700 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2720 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO17;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2730 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2779 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2783 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO18
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2802 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2822 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO18;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2832 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2881 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2885 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO19
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2904 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2924 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO19;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2934 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2983 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2987 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO20
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3006 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3026 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO20;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3036 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3085 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3089 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO21
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3108 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3128 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO21;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3138 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3187 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3191 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO22
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3210 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3230 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO22;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3240 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3289 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3293 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO23
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3312 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3332 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO23;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3342 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3391 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3395 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO24
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3414 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3434 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO24;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3444 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3493 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3497 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO25
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3516 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3536 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO25;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3546 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3595 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3599 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO26
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3618 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3638 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO26;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3648 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3697 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3701 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO27
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3720 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3740 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO27;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3750 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3799 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3803 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO28
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3822 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3842 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO28;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3852 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3901 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3905 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO29
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3924 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3944 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO29;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3954 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4003 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4007 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO30
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4026 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 4046 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO30;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4056 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4105 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4109 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO31
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4128 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 4148 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO31;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4158 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4207 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4211 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO0
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4230 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4250 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO0;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4260 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4309 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4313 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO1
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4332 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4352 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO1;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4362 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4411 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4415 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO2
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4434 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4454 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO2;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4464 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4513 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4517 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO3
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4536 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4556 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO3;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4566 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4615 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4619 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO4
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4638 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4658 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO4;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4668 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4717 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4721 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO5
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4740 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4760 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO5;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4770 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4819 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4823 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO6
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4842 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4862 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO6;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4872 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4921 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4925 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO7
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4944 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4964 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO7;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4974 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5023 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5027 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO8
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5046 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5066 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO8;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5076 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5125 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5129 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO9
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5148 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5168 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO9;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5178 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5227 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5231 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO10
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5250 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5270 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO10;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5280 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5329 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5333 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO11
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5352 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5372 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO11;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5382 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5431 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5435 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO12
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5454 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5474 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO12;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5484 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5533 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5537 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO13
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5556 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5576 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO13;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5586 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5635 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5639 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO14
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5658 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5678 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO14;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5688 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5737 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5741 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO15
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5760 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5780 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO15;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5790 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5839 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5843 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO16
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5862 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5882 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO16;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5892 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5941 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5945 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO17
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5964 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5984 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO17;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5994 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6043 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6047 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO18
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6066 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6086 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO18;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6096 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6145 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6149 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO19
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6168 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6188 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO19;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6198 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6247 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6251 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO20
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6270 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6290 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO20;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6300 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6349 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6353 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO21
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6372 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6392 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO21;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6402 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6451 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6455 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO22
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6474 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6494 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO22;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6504 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6553 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6557 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO23
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6576 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6596 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO23;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6606 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6655 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6659 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO24
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6678 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6698 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO24;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6708 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6757 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6761 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO25
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6780 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6800 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO25;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6810 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6859 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6863 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO26
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6882 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6902 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO26;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6912 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6961 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6965 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO27
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6984 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7004 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO27;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7014 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7063 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7067 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO28
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7086 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7106 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO28;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7116 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7165 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7169 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO29
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7188 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7208 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO29;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7218 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7267 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7271 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO30
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7290 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7310 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO30;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7320 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7369 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7373 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO31
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7392 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7412 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO31;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7422 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7471 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7475 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO0
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7494 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7514 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO0;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7524 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7573 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7577 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO1
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7596 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7616 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO1;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7626 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7675 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7679 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO2
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7698 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7718 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO2;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7728 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7777 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7781 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO3
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7800 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7820 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO3;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7830 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7879 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7883 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO4
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7902 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7922 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO4;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7932 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7981 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7985 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO5
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8004 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8024 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO5;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8034 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8083 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8087 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO6
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8106 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8126 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO6;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8136 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8185 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8189 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO7
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8208 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8228 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO7;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8238 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8287 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8291 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO8
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8310 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8330 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO8;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8340 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8389 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8393 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO9
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8412 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8432 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO9;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8442 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8491 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8495 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO10
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8514 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8534 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO10;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8544 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8593 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8597 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO11
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8616 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8636 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO11;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8646 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8695 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8699 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO12
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8718 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8738 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO12;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8748 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8797 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8801 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO13
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8820 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8840 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO13;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8850 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8899 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8903 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO14
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8922 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8942 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO14;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8952 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9001 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9005 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO15
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9024 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9044 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO15;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9054 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9103 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9107 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO16
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9126 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9146 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO16;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9156 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9205 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9209 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO17
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9228 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9248 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO17;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9258 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9307 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9311 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO18
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9330 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9350 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO18;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9360 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9409 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9413 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO19
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9432 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9452 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO19;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9462 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9511 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9515 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO20
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9534 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9554 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO20;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9564 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9613 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9617 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO21
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9636 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9656 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO21;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9666 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9715 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9719 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO22
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9738 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9758 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO22;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9768 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9817 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9821 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO23
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9840 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9860 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO23;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9870 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9919 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9923 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO24
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9942 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9962 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO24;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9972 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10021 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO25
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10044 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10064 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO25;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10074 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10123 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10127 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO26
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10146 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10166 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO26;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10176 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10225 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10229 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO27
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10248 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10268 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO27;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10278 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10327 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10331 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO28
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10350 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10370 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO28;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10380 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10429 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10433 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO29
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10452 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10472 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO29;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10482 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10531 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10535 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO30
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10554 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10574 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO30;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10584 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10633 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10637 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO31
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10656 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10676 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO31;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10686 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10735 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10739 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO0
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10758 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 10778 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO0;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10788 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10837 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10841 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO1
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10860 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 10880 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO1;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10890 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10939 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10943 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO2
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10962 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 10982 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO2;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10992 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11041 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11045 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO3
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11064 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11084 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO3;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11094 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11143 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11147 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO4
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11166 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11186 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO4;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11196 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11245 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11249 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO5
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11268 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11288 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO5;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11298 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11347 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11351 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO6
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11370 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11390 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO6;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11400 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11449 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11453 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO7
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11472 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11492 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO7;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11502 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11551 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11555 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO8
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11574 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11594 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO8;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11604 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11653 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11657 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO9
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11676 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11696 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO9;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11706 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11755 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11759 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO10
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11778 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11798 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO10;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11808 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11857 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11861 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO11
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11880 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11900 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO11;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11910 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11959 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11963 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO12
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11982 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12002 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO12;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12012 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12061 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12065 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO13
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12084 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12104 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO13;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12114 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12163 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12167 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO14
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12186 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12206 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO14;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12216 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12265 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12269 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO15
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12288 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12308 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO15;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12318 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12367 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12371 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO16
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12390 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12410 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO16;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12420 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12469 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12473 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO17
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12492 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12512 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO17;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12522 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12571 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12575 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO18
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12594 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12614 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO18;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12624 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12673 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12677 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO19
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12696 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12716 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO19;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12726 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12775 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12779 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO20
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12798 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12818 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO20;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12828 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12877 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12881 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO21
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12900 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12920 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO21;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12930 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12979 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12983 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO22
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13002 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13022 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO22;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13032 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13081 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13085 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO23
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13104 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13124 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO23;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13134 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13183 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13187 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO24
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13206 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13226 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO24;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13236 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13285 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13289 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO25
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13308 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13328 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO25;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13338 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13387 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13391 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO26
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13410 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13430 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO26;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13440 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13489 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13493 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO27
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13512 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13532 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO27;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13542 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13591 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13595 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO28
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13614 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13634 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO28;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13644 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13693 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13697 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO29
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13716 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13736 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO29;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13746 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13795 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13799 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO30
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13818 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13838 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO30;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13848 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13897 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13901 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO31
#line 1518 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13920 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1518 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1518 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13940 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO31;

    /* semantic routine */
#line 1518 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13950 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13999 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14003 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_System_Call(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_System_Call
#undef MY_PREFIX
#define MY_PREFIX System_Call
#line 1630 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode System Call\n"));
#line 14022 "icache.c"

  /* validate: 0.17,6./,11./,16./,30.1,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x44000002)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.17,6./,11./,16./,30.1,31./ #define get-values-from-icache|put-values-in-icache */
#line 14031 "icache.c"

#define insn cache_entry->crack.SC.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_System_Call;

    /* semantic routine */
#line 1630 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics System Call\n"));
#line 14041 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_System_Call, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1630 "./ppc-instructions"
    {
      if (CURRENT_MODEL_ISSUE > 0)
        model_serialize(MY_INDEX, cpu_model(processor));
      system_call_interrupt(processor, cia);
    }
#line 14056 "icache.c"
    return nia;
    /* extraction: 0.17,6./,11./,16./,30.1,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14059 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA0_LK0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA0_LK0
#line 1505 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14078 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1505 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1505 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 14093 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA0_LK0;

    /* semantic routine */
#line 1505 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14103 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1505 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14122 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14125 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA0_LK1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA0_LK1
#line 1505 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14144 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1505 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1505 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 14159 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA0_LK1;

    /* semantic routine */
#line 1505 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14169 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1505 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14188 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14191 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA1_LK0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA1_LK0
#line 1505 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14210 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1505 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1505 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 14225 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA1_LK0;

    /* semantic routine */
#line 1505 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14235 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1505 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14254 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14257 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA1_LK1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA1_LK1
#line 1505 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14276 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1505 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1505 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 14291 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA1_LK1;

    /* semantic routine */
#line 1505 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14301 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1505 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14320 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14323 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_Condition_Register_Field(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_Condition_Register_Field
#undef MY_PREFIX
#define MY_PREFIX Move_Condition_Register_Field
#line 1709 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move Condition Register Field\n"));
#line 14342 "icache.c"

  /* validate: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc63ffff) != 0x4c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.XL.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.XL.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 1709 "./ppc-instructions"
    unsigned const BFA UNUSED = cache_entry->crack.XL.BFA = EXTRACTED32(instruction, 11, 13); /* default in icache - extraction */
#line 14357 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_Condition_Register_Field;

    /* semantic routine */
#line 1709 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move Condition Register Field\n"));
#line 14367 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_Condition_Register_Field, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1709 "./ppc-instructions"
    {
      MBLIT32(CR, 4*BF, 4*BF+3, EXTRACTED32(CR, 4*BFA, 4*BFA+3));
      PPC_INSN_CR(BF_BITMASK, 1 << BFA);
    }
#line 14381 "icache.c"
    return nia;
    /* extraction: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14384 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Link_Register_LK0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Link_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Link_Register_LK0
#line 1561 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Link Register\n"));
#line 14403 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.16,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000020)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1561 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1561 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 14419 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Link_Register_LK0;

    /* semantic routine */
#line 1561 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Link Register\n"));
#line 14429 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Link_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1561 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != BO_3_);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (ctr_ok && cond_ok) {
        NIA = IEA(LR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 14470 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14474 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Link_Register_LK1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Link_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Link_Register_LK1
#line 1561 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Link Register\n"));
#line 14493 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.16,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000020)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1561 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1561 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 14509 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Link_Register_LK1;

    /* semantic routine */
#line 1561 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Link Register\n"));
#line 14519 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Link_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1561 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != BO_3_);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (ctr_ok && cond_ok) {
        NIA = IEA(LR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 14560 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14564 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_NOR(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_NOR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_NOR
#line 1674 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register NOR\n"));
#line 14583 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.33,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000042)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.33,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14606 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_NOR;

    /* semantic routine */
#line 1674 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register NOR\n"));
#line 14616 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1674 "./ppc-instructions"
    {
      BLIT32(CR, BT, !(CR_BA_ || CR_BB_));
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14630 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.33,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14635 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Return_From_Interrupt(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Return_From_Interrupt
#undef MY_PREFIX
#define MY_PREFIX Return_From_Interrupt
#line 4862 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Return From Interrupt\n"));
#line 14654 "icache.c"

  /* validate: 0.19,6./,11./,16./,21.50,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x4c000064)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6./,11./,16./,21.50,31./ #define get-values-from-icache|put-values-in-icache */
#line 14663 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Return_From_Interrupt;

    /* semantic routine */
#line 4862 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Return From Interrupt\n"));
#line 14673 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Return_From_Interrupt, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4862 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor)) {
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      }
      else {
        MSR = (MASKED(SRR1, 0, 32)
               | MASKED(SRR1, 37, 41)
               | MASKED(SRR1, 48, 63));
        NIA = MASKED(SRR0, 0, 61);
        cpu_synchronize_context(processor, cia);
        check_masked_interrupts(processor);
      }
    }
#line 14697 "icache.c"
    return nia;
    /* extraction: 0.19,6./,11./,16./,21.50,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14700 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_AND_with_Complement(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_AND_with_Complement
#line 1690 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register AND with Complement\n"));
#line 14719 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.129,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000102)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.129,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14742 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_AND_with_Complement;

    /* semantic routine */
#line 1690 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register AND with Complement\n"));
#line 14752 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1690 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ && !CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14766 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.129,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14771 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Instruction_Synchronize(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Instruction_Synchronize
#undef MY_PREFIX
#define MY_PREFIX Instruction_Synchronize
#line 4775 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Instruction Synchronize\n"));
#line 14790 "icache.c"

  /* validate: 0.19,6./,11./,16./,21.150,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x4c00012c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6./,11./,16./,21.150,31./ #define get-values-from-icache|put-values-in-icache */
#line 14799 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Instruction_Synchronize;

    /* semantic routine */
#line 4775 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Instruction Synchronize\n"));
#line 14809 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Instruction_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4775 "./ppc-instructions"
    {
      cpu_synchronize_context(processor, cia);
      PPC_INSN_INT(0, 0, 0);
    }
#line 14823 "icache.c"
    return nia;
    /* extraction: 0.19,6./,11./,16./,21.150,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14826 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_XOR(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_XOR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_XOR
#line 1658 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register XOR\n"));
#line 14845 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.193,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000182)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.193,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14868 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_XOR;

    /* semantic routine */
#line 1658 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register XOR\n"));
#line 14878 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1658 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ != CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14892 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.193,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14897 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_NAND(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_NAND
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_NAND
#line 1666 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register NAND\n"));
#line 14916 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.225,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c0001c2)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.225,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14939 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_NAND;

    /* semantic routine */
#line 1666 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register NAND\n"));
#line 14949 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1666 "./ppc-instructions"
    {
      BLIT32(CR, BT, !(CR_BA_ && CR_BB_));
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14963 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.225,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14968 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_AND(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_AND
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_AND
#line 1642 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register AND\n"));
#line 14987 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.257,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000202)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.257,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15010 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_AND;

    /* semantic routine */
#line 1642 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register AND\n"));
#line 15020 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1642 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ && CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15034 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.257,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15039 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_Equivalent(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_Equivalent
#line 1682 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register Equivalent\n"));
#line 15058 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.289,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000242)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.289,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15081 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_Equivalent;

    /* semantic routine */
#line 1682 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register Equivalent\n"));
#line 15091 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1682 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ == CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15105 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.289,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15110 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_OR_with_Complement(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_OR_with_Complement
#line 1698 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register OR with Complement\n"));
#line 15129 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.417,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000342)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.417,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15152 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_OR_with_Complement;

    /* semantic routine */
#line 1698 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register OR with Complement\n"));
#line 15162 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1698 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ || !CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15176 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.417,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15181 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_OR(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_OR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_OR
#line 1650 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register OR\n"));
#line 15200 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.449,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000382)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.449,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15223 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_OR;

    /* semantic routine */
#line 1650 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register OR\n"));
#line 15233 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1650 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ || CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15247 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.449,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15252 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Count_Register_LK0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Count_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Count_Register_LK0
#line 1596 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Count Register\n"));
#line 15271 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.528,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000420)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1596 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1596 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 15287 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Count_Register_LK0;

    /* semantic routine */
#line 1596 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Count Register\n"));
#line 15297 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Count_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1596 "./ppc-instructions"
    {
      int cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (cond_ok) {
        NIA = IEA(CTR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 15334 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 15338 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Count_Register_LK1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Count_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Count_Register_LK1
#line 1596 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Count Register\n"));
#line 15357 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.528,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000420)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1596 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1596 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 15373 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Count_Register_LK1;

    /* semantic routine */
#line 1596 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Count Register\n"));
#line 15383 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Count_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1596 "./ppc-instructions"
    {
      int cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (cond_ok) {
        NIA = IEA(CTR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 15420 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 15424 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0(cpu *processor,
                                                       instruction_word instruction,
                                                       unsigned_word cia,
                                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_Mask_Insert
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0
#line 3231 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then Mask Insert\n"));
#line 15443 "icache.c"

  /* validate: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x50000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3231 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3231 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3231 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3231 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 15473 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0;

    /* semantic routine */
#line 3231 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then Mask Insert\n"));
#line 15483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_Mask_Insert, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3231 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = (r & m) | (*rA & ~m);
      *rA = result;
      ITRACE(trace_alu, (": n=%ld *rS=0x%lx r=0x%lx m=0x%lx result=0x%lx\n",
                         n, (unsigned long)*rS, (unsigned long)r, (unsigned long)m,
      		   (unsigned long)result));
      CR0_COMPARE(result, 0, Rc);
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15505 "icache.c"
    return nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15509 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1(cpu *processor,
                                                       instruction_word instruction,
                                                       unsigned_word cia,
                                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_Mask_Insert
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1
#line 3231 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then Mask Insert\n"));
#line 15528 "icache.c"

  /* validate: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x50000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3231 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3231 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3231 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3231 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 15558 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1;

    /* semantic routine */
#line 3231 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then Mask Insert\n"));
#line 15568 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_Mask_Insert, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3231 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = (r & m) | (*rA & ~m);
      *rA = result;
      ITRACE(trace_alu, (": n=%ld *rS=0x%lx r=0x%lx m=0x%lx result=0x%lx\n",
                         n, (unsigned long)*rS, (unsigned long)r, (unsigned long)m,
      		   (unsigned long)result));
      CR0_COMPARE(result, 0, Rc);
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15590 "icache.c"
    return nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15594 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0(cpu *processor,
                                                         instruction_word instruction,
                                                         unsigned_word cia,
                                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0
#line 3178 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then AND with Mask\n"));
#line 15613 "icache.c"

  /* validate: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x54000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3178 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3178 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3178 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3178 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 15643 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0;

    /* semantic routine */
#line 3178 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then AND with Mask\n"));
#line 15653 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3178 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 s = *rS;
      unsigned32 r = ROTL32(s, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
      ITRACE(trace_alu,
             ("n=%ld, s=0x%lx, r=0x%lx, m=0x%lx, result=0x%lx, cr=0x%lx\n",
              n, (unsigned long)s, (unsigned long)r, (unsigned long)m,
      	(unsigned long)result, (unsigned long)CR));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15677 "icache.c"
    return nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15681 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1(cpu *processor,
                                                         instruction_word instruction,
                                                         unsigned_word cia,
                                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1
#line 3178 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then AND with Mask\n"));
#line 15700 "icache.c"

  /* validate: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x54000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3178 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3178 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3178 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3178 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 15730 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1;

    /* semantic routine */
#line 3178 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then AND with Mask\n"));
#line 15740 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3178 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 s = *rS;
      unsigned32 r = ROTL32(s, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
      ITRACE(trace_alu,
             ("n=%ld, s=0x%lx, r=0x%lx, m=0x%lx, result=0x%lx, cr=0x%lx\n",
              n, (unsigned long)s, (unsigned long)r, (unsigned long)m,
      	(unsigned long)result, (unsigned long)CR));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15764 "icache.c"
    return nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15768 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_then_AND_with_Mask_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_then_AND_with_Mask_Rc0
#line 3210 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word then AND with Mask\n"));
#line 15787 "icache.c"

  /* validate: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x5c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.M.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.M.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.M.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3210 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3210 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3210 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 15821 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_then_AND_with_Mask_Rc0;

    /* semantic routine */
#line 3210 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word then AND with Mask\n"));
#line 15831 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3210 "./ppc-instructions"
    {
      long n = MASKED(*rB, 59, 63);
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
    }
#line 15849 "icache.c"
    return nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15853 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_then_AND_with_Mask_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_then_AND_with_Mask_Rc1
#line 3210 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word then AND with Mask\n"));
#line 15872 "icache.c"

  /* validate: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x5c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.M.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.M.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.M.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3210 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3210 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3210 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 15906 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_then_AND_with_Mask_Rc1;

    /* semantic routine */
#line 3210 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word then AND with Mask\n"));
#line 15916 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3210 "./ppc-instructions"
    {
      long n = MASKED(*rB, 59, 63);
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
    }
#line 15934 "icache.c"
    return nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15938 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Immediate(cpu *processor,
                    instruction_word instruction,
                    unsigned_word cia,
                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_Immediate
#undef MY_PREFIX
#define MY_PREFIX OR_Immediate
#line 2974 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR Immediate\n"));
#line 15957 "icache.c"

  /* validate: 0.24,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x60000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.24,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2974 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 15981 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Immediate;

    /* semantic routine */
#line 2974 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR Immediate\n"));
#line 15991 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2974 "./ppc-instructions"
    {
      *rA = *rS | UI;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16006 "icache.c"
    return nia;
    /* extraction: 0.24,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16010 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Immediate_Shifted(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX OR_Immediate_Shifted
#line 2983 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR Immediate Shifted\n"));
#line 16029 "icache.c"

  /* validate: 0.25,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x64000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.25,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2983 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16053 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Immediate_Shifted;

    /* semantic routine */
#line 2983 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR Immediate Shifted\n"));
#line 16063 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2983 "./ppc-instructions"
    {
      *rA = *rS | (UI << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16078 "icache.c"
    return nia;
    /* extraction: 0.25,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16082 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Immediate(cpu *processor,
                     instruction_word instruction,
                     unsigned_word cia,
                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR_Immediate
#undef MY_PREFIX
#define MY_PREFIX XOR_Immediate
#line 2992 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR Immediate\n"));
#line 16101 "icache.c"

  /* validate: 0.26,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x68000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.26,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2992 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16125 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Immediate;

    /* semantic routine */
#line 2992 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR Immediate\n"));
#line 16135 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2992 "./ppc-instructions"
    {
      *rA = *rS ^ UI;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16150 "icache.c"
    return nia;
    /* extraction: 0.26,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16154 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Immediate_Shifted(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX XOR_Immediate_Shifted
#line 3001 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR Immediate Shifted\n"));
#line 16173 "icache.c"

  /* validate: 0.27,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x6c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.27,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3001 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16197 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Immediate_Shifted;

    /* semantic routine */
#line 3001 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR Immediate Shifted\n"));
#line 16207 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3001 "./ppc-instructions"
    {
      *rA = *rS ^ (UI << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16222 "icache.c"
    return nia;
    /* extraction: 0.27,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16226 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Immediate(cpu *processor,
                     instruction_word instruction,
                     unsigned_word cia,
                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_Immediate
#undef MY_PREFIX
#define MY_PREFIX AND_Immediate
#line 2954 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND Immediate\n"));
#line 16245 "icache.c"

  /* validate: 0.28,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x70000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.28,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2954 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16269 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Immediate;

    /* semantic routine */
#line 2954 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND Immediate\n"));
#line 16279 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2954 "./ppc-instructions"
    {
      *rA = *rS & UI;
      CR0_COMPARE(*rA, 0, 1/*Rc*/);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 1/*Rc*/);
    }
#line 16295 "icache.c"
    return nia;
    /* extraction: 0.28,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16299 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Immediate_Shifted(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX AND_Immediate_Shifted
#line 2964 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND Immediate Shifted\n"));
#line 16318 "icache.c"

  /* validate: 0.29,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x74000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.29,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2964 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16342 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Immediate_Shifted;

    /* semantic routine */
#line 2964 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND Immediate Shifted\n"));
#line 16352 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2964 "./ppc-instructions"
    {
      *rA = *rS & (UI << 16);
      CR0_COMPARE(*rA, 0, 1/*Rc*/);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 1/*Rc*/);
    }
#line 16368 "icache.c"
    return nia;
    /* extraction: 0.29,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16372 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare
#undef MY_PREFIX
#define MY_PREFIX Compare
#line 2810 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare\n"));
#line 16391 "icache.c"

  /* validate: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc4007ff) != 0x7c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2810 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 16419 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare;

    /* semantic routine */
#line 2810 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare\n"));
#line 16429 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2810 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        signed_word a;
        signed_word b;
        if (L == 0) {
          a = EXTENDED(*rA);
          b = EXTENDED(*rB);
        }
        else {
          a = *rA;
          b = *rB;
        }
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK | RB_BITMASK, BF_BITMASK);
    }
#line 16458 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16462 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Trap_Word(cpu *processor,
                 instruction_word instruction,
                 unsigned_word cia,
                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Trap_Word
#undef MY_PREFIX
#define MY_PREFIX Trap_Word
#line 2930 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Trap Word\n"));
#line 16481 "icache.c"

  /* validate: 0.31,6.TO,11.RA,16.RB,21.4,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c000008)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.TO,11.RA,16.RB,21.4,31./ #define get-values-from-icache|put-values-in-icache */
#line 2930 "./ppc-instructions"
    unsigned const TO UNUSED = cache_entry->crack.X.TO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 16505 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Trap_Word;

    /* semantic routine */
#line 2930 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Trap Word\n"));
#line 16515 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Trap_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2930 "./ppc-instructions"
    {
      signed_word a = EXTENDED(*rA);
      signed_word b = EXTENDED(*rB);
      if (TO == 12 && rA == rB) {
        ITRACE(trace_breakpoint, ("breakpoint\n"));
        cpu_halt(processor, cia, was_trap, 0);
      }
      else if ((a < b && TO_0_)
          || (a > b && TO_1_)
          || (a == b && TO_2_)
          || ((unsigned_word)a < (unsigned_word)b && TO_3_)
          || ((unsigned_word)a > (unsigned_word)b && TO_4_)
          )
        program_interrupt(processor, cia,
                          trap_program_interrupt);
    }
#line 16541 "icache.c"
    return nia;
    /* extraction: 0.31,6.TO,11.RA,16.RB,21.4,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16545 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE0_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE0_Rc0
#line 2596 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 16564 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2596 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2596 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 16596 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE0_Rc0;

    /* semantic routine */
#line 2596 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 16606 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2596 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16625 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16629 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE0_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE0_Rc1
#line 2596 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 16648 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2596 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2596 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 16680 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE0_Rc1;

    /* semantic routine */
#line 2596 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 16690 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2596 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16709 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16713 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE0_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE0_Rc0
#line 2586 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 16732 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2586 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2586 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 16764 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE0_Rc0;

    /* semantic routine */
#line 2586 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 16774 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2586 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16790 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16794 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE0_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE0_Rc1
#line 2586 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 16813 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2586 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2586 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 16845 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE0_Rc1;

    /* semantic routine */
#line 2586 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 16855 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2586 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16871 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16875 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Unsigned_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Unsigned_Rc0
#line 2730 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word Unsigned\n"));
#line 16894 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000016)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2730 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 16924 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Unsigned_Rc0;

    /* semantic routine */
#line 2730 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word Unsigned\n"));
#line 16934 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2730 "./ppc-instructions"
    {
      unsigned64 a = (unsigned32)(*rA);
      unsigned64 b = (unsigned32)(*rB);
      unsigned64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16953 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16957 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Unsigned_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Unsigned_Rc1
#line 2730 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word Unsigned\n"));
#line 16976 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000016)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2730 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17006 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Unsigned_Rc1;

    /* semantic routine */
#line 2730 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word Unsigned\n"));
#line 17016 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2730 "./ppc-instructions"
    {
      unsigned64 a = (unsigned32)(*rA);
      unsigned64 b = (unsigned32)(*rB);
      unsigned64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 17035 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17039 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Condition_Register(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Condition_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Condition_Register
#line 3453 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Condition Register\n"));
#line 17058 "icache.c"

  /* validate: 0.31,6.RT,11./,16./,21.19,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c000026)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16./,21.19,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 17073 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Condition_Register;

    /* semantic routine */
#line 3453 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Condition Register\n"));
#line 17083 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Condition_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3453 "./ppc-instructions"
    {
      *rT = (unsigned32)CR;
      PPC_INSN_MFCR(RT_BITMASK);
    }
#line 17097 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16./,21.19,31./ #undef get-values-from-icache|put-values-in-icache */
#line 17100 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_And_Reserve_Indexed(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_And_Reserve_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_And_Reserve_Indexed
#line 2422 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word And Reserve Indexed\n"));
#line 17119 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.20,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.20,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17147 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_And_Reserve_Indexed;

    /* semantic routine */
#line 2422 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word And Reserve Indexed\n"));
#line 17157 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_And_Reserve_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2422 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      RESERVE = 1;
      RESERVE_ADDR = real_addr(EA, 1/*is-read?*/);
      RESERVE_DATA = MEM(unsigned, EA, 4);
      *rT = RESERVE_DATA;
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 17179 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.20,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17183 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_Indexed(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_Indexed
#line 1898 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero Indexed\n"));
#line 17202 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.23,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.23,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17230 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_Indexed;

    /* semantic routine */
#line 1898 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero Indexed\n"));
#line 17240 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1898 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 4);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 17259 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.23,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17263 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Left_Word_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Left_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Left_Word_Rc0
#line 3250 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Left Word\n"));
#line 17282 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3250 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17312 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Left_Word_Rc0;

    /* semantic routine */
#line 3250 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Left Word\n"));
#line 17322 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Left_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3250 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source << n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu,
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 17347 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17351 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Left_Word_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Left_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Left_Word_Rc1
#line 3250 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Left Word\n"));
#line 17370 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3250 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17400 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Left_Word_Rc1;

    /* semantic routine */
#line 3250 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Left Word\n"));
#line 17410 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Left_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3250 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source << n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu,
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 17435 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17439 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Count_Leading_Zeros_Word_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Count_Leading_Zeros_Word
#undef MY_PREFIX
#define MY_PREFIX Count_Leading_Zeros_Word_Rc0
#line 3129 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Count Leading Zeros Word\n"));
#line 17458 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3129 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17482 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Count_Leading_Zeros_Word_Rc0;

    /* semantic routine */
#line 3129 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Count Leading Zeros Word\n"));
#line 17492 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Count_Leading_Zeros_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3129 "./ppc-instructions"
    {
      int count = 0;
      unsigned32 mask = BIT32(0);
      unsigned32 source = *rS;
      while (!(source & mask) && mask != 0) {
        mask >>= 1;
        count++;
      }
      *rA = count;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      CR0_COMPARE(count, 0, Rc); /* FIXME - is this correct */
    }
#line 17514 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17518 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Count_Leading_Zeros_Word_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Count_Leading_Zeros_Word
#undef MY_PREFIX
#define MY_PREFIX Count_Leading_Zeros_Word_Rc1
#line 3129 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Count Leading Zeros Word\n"));
#line 17537 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3129 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17561 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Count_Leading_Zeros_Word_Rc1;

    /* semantic routine */
#line 3129 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Count Leading Zeros Word\n"));
#line 17571 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Count_Leading_Zeros_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3129 "./ppc-instructions"
    {
      int count = 0;
      unsigned32 mask = BIT32(0);
      unsigned32 source = *rS;
      while (!(source & mask) && mask != 0) {
        mask >>= 1;
        count++;
      }
      *rA = count;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      CR0_COMPARE(count, 0, Rc); /* FIXME - is this correct */
    }
#line 17593 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17597 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND
#undef MY_PREFIX
#define MY_PREFIX AND_Rc0
#line 3010 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND\n"));
#line 17616 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3010 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17646 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Rc0;

    /* semantic routine */
#line 3010 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND\n"));
#line 17656 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3010 "./ppc-instructions"
    {
      *rA = *rS & *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 17672 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17676 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND
#undef MY_PREFIX
#define MY_PREFIX AND_Rc1
#line 3010 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND\n"));
#line 17695 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3010 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17725 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Rc1;

    /* semantic routine */
#line 3010 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND\n"));
#line 17735 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3010 "./ppc-instructions"
    {
      *rA = *rS & *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 17751 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17755 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Logical(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Logical
#undef MY_PREFIX
#define MY_PREFIX Compare_Logical
#line 2852 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare Logical\n"));
#line 17774 "icache.c"

  /* validate: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc4007ff) != 0x7c000040)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2852 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17802 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Logical;

    /* semantic routine */
#line 2852 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare Logical\n"));
#line 17812 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Logical, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2852 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        unsigned_word a;
        unsigned_word b;
        if (L == 0) {
          a = MASKED(*rA, 32, 63);
          b = MASKED(*rB, 32, 63);
        }
        else {
          a = *rA;
          b = *rB;
        }
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK | RB_BITMASK, BF_BITMASK);
    }
#line 17841 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17845 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE0_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE0_Rc0
#line 2542 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 17864 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2542 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2542 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17896 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE0_Rc0;

    /* semantic routine */
#line 2542 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 17906 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2542 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 17924 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17928 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE0_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE0_Rc1
#line 2542 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 17947 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2542 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2542 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17979 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE0_Rc1;

    /* semantic routine */
#line 2542 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 17989 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2542 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18007 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18011 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Store(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Store
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Store
#line 4812 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Store\n"));
#line 18030 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.54,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c00006c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.54,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18052 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Store;

    /* semantic routine */
#line 4812 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Store\n"));
#line 18062 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Store, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4812 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Store\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 18076 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.54,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18080 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_with_Update_Indexed(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_with_Update_Indexed
#line 1925 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero with Update Indexed\n"));
#line 18099 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.55,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00006e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.55,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18127 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1925 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero with Update Indexed\n"));
#line 18137 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1925 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 4);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 18157 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.55,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18161 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_with_Complement_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX AND_with_Complement_Rc0
#line 3070 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND with Complement\n"));
#line 18180 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000078)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3070 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18210 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_with_Complement_Rc0;

    /* semantic routine */
#line 3070 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND with Complement\n"));
#line 18220 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3070 "./ppc-instructions"
    {
      *rA = *rS & ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 18236 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18240 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_with_Complement_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX AND_with_Complement_Rc1
#line 3070 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND with Complement\n"));
#line 18259 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000078)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3070 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 18289 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_with_Complement_Rc1;

    /* semantic routine */
#line 3070 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND with Complement\n"));
#line 18299 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3070 "./ppc-instructions"
    {
      *rA = *rS & ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 18315 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18319 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Rc0(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Rc0
#line 2715 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word\n"));
#line 18338 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000096)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2715 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18368 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Rc0;

    /* semantic routine */
#line 2715 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word\n"));
#line 18378 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2715 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18397 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18401 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Rc1(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Rc1
#line 2715 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word\n"));
#line 18420 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000096)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2715 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 18450 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Rc1;

    /* semantic routine */
#line 2715 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word\n"));
#line 18460 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2715 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18479 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18483 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Machine_State_Register(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Machine_State_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Machine_State_Register
#line 4899 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Machine State Register\n"));
#line 18502 "icache.c"

  /* validate: 0.31,6.RT,11./,16./,21.83,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c0000a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16./,21.83,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 18517 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Machine_State_Register;

    /* semantic routine */
#line 4899 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Machine State Register\n"));
#line 18527 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Machine_State_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4899 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        *rT = MSR;
        check_masked_interrupts(processor);
      }
    }
#line 18546 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16./,21.83,31./ #undef get-values-from-icache|put-values-in-icache */
#line 18549 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Flush(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Flush
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Flush
#line 4820 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Flush\n"));
#line 18568 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.86,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0000ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.86,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18590 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Flush;

    /* semantic routine */
#line 4820 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Flush\n"));
#line 18600 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Flush, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4820 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Flush\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 18614 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.86,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18618 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_Indexed(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_Indexed
#line 1736 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero Indexed\n"));
#line 18637 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.87,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0000ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.87,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18665 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_Indexed;

    /* semantic routine */
#line 1736 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero Indexed\n"));
#line 18675 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1736 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 1);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 18694 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.87,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18698 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE0_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE0_Rc0
#line 2676 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 18717 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2676 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2676 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18743 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE0_Rc0;

    /* semantic routine */
#line 2676 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 18753 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2676 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 18770 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18774 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE0_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE0_Rc1
#line 2676 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 18793 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2676 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2676 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 18819 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE0_Rc1;

    /* semantic routine */
#line 2676 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 18829 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2676 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 18846 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18850 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_with_Update_Indexed(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_with_Update_Indexed
#line 1763 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero with Update Indexed\n"));
#line 18869 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.119,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0000ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.119,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18897 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1763 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero with Update Indexed\n"));
#line 18907 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1763 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 1);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 18927 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.119,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18931 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NOR_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NOR
#undef MY_PREFIX
#define MY_PREFIX NOR_Rc0
#line 3050 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NOR\n"));
#line 18950 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0000f8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3050 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18980 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NOR_Rc0;

    /* semantic routine */
#line 3050 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NOR\n"));
#line 18990 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3050 "./ppc-instructions"
    {
      *rA = ~(*rS | *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 19006 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19010 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NOR_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NOR
#undef MY_PREFIX
#define MY_PREFIX NOR_Rc1
#line 3050 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NOR\n"));
#line 19029 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0000f8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3050 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19059 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NOR_Rc1;

    /* semantic routine */
#line 3050 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NOR\n"));
#line 19069 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3050 "./ppc-instructions"
    {
      *rA = ~(*rS | *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 19085 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19089 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE0_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE0_Rc0
#line 2620 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 19108 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2620 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2620 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 19140 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE0_Rc0;

    /* semantic routine */
#line 2620 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 19150 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2620 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19168 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19172 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE0_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE0_Rc1
#line 2620 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 19191 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2620 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2620 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19223 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE0_Rc1;

    /* semantic routine */
#line 2620 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 19233 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2620 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19251 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19255 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE0_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE0_Rc0
#line 2609 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 19274 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2609 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2609 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 19306 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE0_Rc0;

    /* semantic routine */
#line 2609 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 19316 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2609 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19333 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19337 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE0_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE0_Rc1
#line 2609 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 19356 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2609 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2609 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19388 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE0_Rc1;

    /* semantic routine */
#line 2609 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 19398 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2609 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19415 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19419 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_Fields(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_Fields
#line 3430 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Condition Register Fields\n"));
#line 19438 "icache.c"

  /* validate: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc100fff) != 0x7c000120)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3430 "./ppc-instructions"
    unsigned const FXM UNUSED = cache_entry->crack.XFX.FXM = EXTRACTED32(instruction, 12, 19); /* default in icache - extraction */
#line 19455 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_Fields;

    /* semantic routine */
#line 3430 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Condition Register Fields\n"));
#line 19465 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3430 "./ppc-instructions"
    {
      if (FXM == 0xff) {
        CR = *rS;
      }
      else {
        unsigned_word mask = 0;
        unsigned_word f;
        for (f = 0; f < 8; f++) {
          if (FXM & (0x80 >> f))
            mask |= (0xf << 4*(7-f));
        }
        CR = (MASKED(*rS, 32, 63) & mask) | (CR & ~mask);
      }
      PPC_INSN_MTCR(RS_BITMASK, FXM);
    }
#line 19490 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ #undef get-values-from-icache|put-values-in-icache */
#line 19493 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Machine_State_Register(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Machine_State_Register
#undef MY_PREFIX
#define MY_PREFIX Move_To_Machine_State_Register
#line 4886 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To Machine State Register\n"));
#line 19512 "icache.c"

  /* validate: 0.31,6.RS,11./,16./,21.146,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c000124)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,16./,21.146,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 19527 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Machine_State_Register;

    /* semantic routine */
#line 4886 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To Machine State Register\n"));
#line 19537 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Machine_State_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4886 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        MSR = *rS;
        check_masked_interrupts(processor);
      }
    }
#line 19556 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,16./,21.146,31./ #undef get-values-from-icache|put-values-in-icache */
#line 19559 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_Conditional_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_Conditional_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_Conditional_Indexed
#line 2450 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word Conditional Indexed\n"));
#line 19578 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.150,31.1 */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00012d)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.150,31.1 #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19606 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_Conditional_Indexed;

    /* semantic routine */
#line 2450 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word Conditional Indexed\n"));
#line 19616 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_Conditional_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2450 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      if (RESERVE) {
        if (RESERVE_ADDR == real_addr(EA, 0/*is-read?*/)
            && /*HACK*/ RESERVE_DATA == MEM(unsigned, EA, 4)) {
          STORE(EA, 4, *rS);
          CR_SET_XER_SO(0, cr_i_zero);
        }
        else {
          /* ment to randomly to store, we never do! */	
          CR_SET_XER_SO(0, 0);
        }
        RESERVE = 0;
      }
      else {
        CR_SET_XER_SO(0, 0);
      }
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 1/*Rc*/);
    }
#line 19649 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.150,31.1 #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19653 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_Indexed(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_Indexed
#line 2125 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word Indexed\n"));
#line 19672 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.151,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00012e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.151,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19700 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_Indexed;

    /* semantic routine */
#line 2125 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word Indexed\n"));
#line 19710 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2125 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 19729 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.151,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19733 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_with_Update_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_with_Update_Indexed
#line 2152 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word with Update Indexed\n"));
#line 19752 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.183,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00016e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.183,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19780 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_with_Update_Indexed;

    /* semantic routine */
#line 2152 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word with Update Indexed\n"));
#line 19790 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2152 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 4, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 19810 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.183,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19814 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE0_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE0_Rc0
#line 2665 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 19833 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2665 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2665 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 19859 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE0_Rc0;

    /* semantic routine */
#line 2665 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 19869 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2665 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 19886 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19890 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE0_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE0_Rc1
#line 2665 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 19909 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2665 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2665 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19935 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE0_Rc1;

    /* semantic routine */
#line 2665 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 19945 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2665 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 19962 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19966 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE0_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE0_Rc0
#line 2655 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 19985 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2655 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2655 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20011 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE0_Rc0;

    /* semantic routine */
#line 2655 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 20021 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2655 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20037 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20041 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE0_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE0_Rc1
#line 2655 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 20060 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2655 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2655 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20086 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE0_Rc1;

    /* semantic routine */
#line 2655 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 20096 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2655 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20112 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20116 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Segment_Register(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Segment_Register
#undef MY_PREFIX
#define MY_PREFIX Move_To_Segment_Register
#line 4932 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To Segment Register\n"));
#line 20135 "icache.c"

  /* validate: 0.31,6.RS,11./,12.SR,16./,21.210,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc10ffff) != 0x7c0001a4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,12.SR,16./,21.210,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 4932 "./ppc-instructions"
    unsigned const SR UNUSED = cache_entry->crack.X.SR = EXTRACTED32(instruction, 12, 15); /* default in icache - extraction */
#line 20152 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Segment_Register;

    /* semantic routine */
#line 4932 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To Segment Register\n"));
#line 20162 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Segment_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4932 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        SEGREG(SR) = *rS;
    }
#line 20179 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,12.SR,16./,21.210,31./ #undef get-values-from-icache|put-values-in-icache */
#line 20182 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_Indexed(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_Indexed
#line 2017 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte Indexed\n"));
#line 20201 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.215,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0001ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.215,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20229 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_Indexed;

    /* semantic routine */
#line 2017 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte Indexed\n"));
#line 20239 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2017 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 1, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 20258 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.215,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20262 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE0_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE0_Rc0
#line 2643 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 20281 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2643 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2643 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20307 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE0_Rc0;

    /* semantic routine */
#line 2643 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 20317 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2643 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20335 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20339 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE0_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE0_Rc1
#line 2643 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 20358 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2643 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2643 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20384 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE0_Rc1;

    /* semantic routine */
#line 2643 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 20394 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2643 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20412 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20416 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE0_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE0_Rc0
#line 2632 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 20435 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2632 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2632 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20461 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE0_Rc0;

    /* semantic routine */
#line 2632 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 20471 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2632 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20488 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20492 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE0_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE0_Rc1
#line 2632 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 20511 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2632 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2632 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20537 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE0_Rc1;

    /* semantic routine */
#line 2632 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 20547 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2632 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20564 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20568 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE0_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE0_Rc0
#line 2698 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 20587 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2698 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2698 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20619 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE0_Rc0;

    /* semantic routine */
#line 2698 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 20629 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2698 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 20650 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20654 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE0_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE0_Rc1
#line 2698 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 20673 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2698 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2698 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20705 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE0_Rc1;

    /* semantic routine */
#line 2698 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 20715 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2698 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 20736 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20740 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Segment_Register_Indirect(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Segment_Register_Indirect
#undef MY_PREFIX
#define MY_PREFIX Move_To_Segment_Register_Indirect
#line 4943 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To Segment Register Indirect\n"));
#line 20759 "icache.c"

  /* validate: 0.31,6.RS,11./,16.RB,21.242,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07ff) != 0x7c0001e4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,16.RB,21.242,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20780 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Segment_Register_Indirect;

    /* semantic routine */
#line 4943 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To Segment Register Indirect\n"));
#line 20790 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Segment_Register_Indirect, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4943 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        SEGREG(EXTRACTED32(*rB, 0, 3)) = *rS;
    }
#line 20807 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,16.RB,21.242,31./ #undef get-values-from-icache|put-values-in-icache */
#line 20810 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Touch_for_Store(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Touch_for_Store
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Touch_for_Store
#line 4796 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Touch for Store\n"));
#line 20829 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.246,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0001ec)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.246,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20851 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Touch_for_Store;

    /* semantic routine */
#line 4796 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Touch for Store\n"));
#line 20861 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Touch_for_Store, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4796 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Touch for Store\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 20875 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.246,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20879 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_with_Update_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_with_Update_Indexed
#line 2044 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte with Update Indexed\n"));
#line 20898 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.247,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0001ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.247,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20926 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_with_Update_Indexed;

    /* semantic routine */
#line 2044 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte with Update Indexed\n"));
#line 20936 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2044 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 1, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 20956 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.247,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20960 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE0_Rc0(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE0_Rc0
#line 2532 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 20979 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2532 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2532 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 21011 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE0_Rc0;

    /* semantic routine */
#line 2532 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 21021 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2532 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 21037 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21041 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE0_Rc1(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE0_Rc1
#line 2532 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 21060 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2532 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2532 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 21092 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE0_Rc1;

    /* semantic routine */
#line 2532 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 21102 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2532 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 21118 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21122 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Touch(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Touch
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Touch
#line 4788 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Touch\n"));
#line 21141 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.278,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c00022c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.278,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21163 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Touch;

    /* semantic routine */
#line 4788 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Touch\n"));
#line 21173 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Touch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4788 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Touch\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 21187 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.278,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21191 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_Indexed
#line 1790 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero Indexed\n"));
#line 21210 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.279,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00022e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.279,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21238 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_Indexed;

    /* semantic routine */
#line 1790 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero Indexed\n"));
#line 21248 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1790 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 21267 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.279,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21271 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Equivalent_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Equivalent_Rc0
#line 3060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Equivalent\n"));
#line 21290 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000238)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 21320 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Equivalent_Rc0;

    /* semantic routine */
#line 3060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Equivalent\n"));
#line 21330 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3060 "./ppc-instructions"
    {
      *rA = ~(*rS ^ *rB); /* A === B */
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21346 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21350 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Equivalent_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Equivalent_Rc1
#line 3060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Equivalent\n"));
#line 21369 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000238)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 21399 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Equivalent_Rc1;

    /* semantic routine */
#line 3060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Equivalent\n"));
#line 21409 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3060 "./ppc-instructions"
    {
      *rA = ~(*rS ^ *rB); /* A === B */
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21425 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21429 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Invalidate_Entry(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Invalidate_Entry
#undef MY_PREFIX
#define MY_PREFIX TLB_Invalidate_Entry
#line 4981 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode TLB Invalidate Entry\n"));
#line 21448 "icache.c"

  /* validate: 0.31,6./,11./,16.RB,21.306,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffff07ff) != 0x7c000264)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16.RB,21.306,31./ #define get-values-from-icache|put-values-in-icache */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21463 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Invalidate_Entry;

    /* semantic routine */
#line 4981 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics TLB Invalidate Entry\n"));
#line 21473 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Invalidate_Entry, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4981 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        int nr = 0;
        cpu *proc;
        while (1) {
          proc = psim_cpu(cpu_system(processor), nr);
          if (proc == NULL) break;
          cpu_page_tlb_invalidate_entry(proc, *rB);
          nr++;
        }
      }
    }
#line 21498 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16.RB,21.306,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21501 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_with_Update_Indexed(cpu *processor,
                                                  instruction_word instruction,
                                                  unsigned_word cia,
                                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_with_Update_Indexed
#line 1817 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero with Update Indexed\n"));
#line 21520 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.311,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00026e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.311,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21548 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1817 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero with Update Indexed\n"));
#line 21558 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1817 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 21578 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.311,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21582 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR
#undef MY_PREFIX
#define MY_PREFIX XOR_Rc0
#line 3030 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR\n"));
#line 21601 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000278)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3030 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 21631 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Rc0;

    /* semantic routine */
#line 3030 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR\n"));
#line 21641 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3030 "./ppc-instructions"
    {
      *rA = *rS ^ *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21657 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21661 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR
#undef MY_PREFIX
#define MY_PREFIX XOR_Rc1
#line 3030 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR\n"));
#line 21680 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000278)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3030 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 21710 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Rc1;

    /* semantic routine */
#line 3030 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR\n"));
#line 21720 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3030 "./ppc-instructions"
    {
      *rA = *rS ^ *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21736 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21740 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_from_Special_Purpose_Register_SPR256(cpu *processor,
                                                 instruction_word instruction,
                                                 unsigned_word cia,
                                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_from_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_from_Special_Purpose_Register_SPR256
#line 3389 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move from Special Purpose Register\n"));
#line 21759 "icache.c"

  /* validate: 0.31,6.RT,11.SPR,21.339,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 3389 "./ppc-instructions"
    unsigned const SPR UNUSED = 256; /* default in icache - constant field */
#line 21776 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_from_Special_Purpose_Register_SPR256;

    /* semantic routine */
#line 3389 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move from Special Purpose Register\n"));
#line 21786 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_from_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3389 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        /* HACK - time base registers need to be calculated */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_dec:
            *rT = cpu_get_decrementer(processor);
            break;
      	case spr_tbrl:
        	  if (is_64bit_implementation) *rT = TB;
        	  else                         *rT = EXTRACTED64(TB, 32, 63);
      	break;
      	case spr_tbru:
        	  if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
      	  else                         *rT = EXTRACTED64(TB, 0, 31);
      	break;
          case spr_tbu:
          case spr_tbl:
            /* NOTE - these SPR's are not readable. Use mftb[ul] */
          default:
            *rT = SPREG(n);
            break;
          }
        }
        else {
          *rT = SPREG(n);
        }
      }
      PPC_INSN_FROM_SPR(RT_BITMASK, n);
    }
#line 21833 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21836 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_from_Special_Purpose_Register_SPR(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_from_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_from_Special_Purpose_Register_SPR
#line 3389 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move from Special Purpose Register\n"));
#line 21855 "icache.c"

  /* validate: 0.31,6.RT,11.SPR,21.339,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 3389 "./ppc-instructions"
    unsigned const SPR UNUSED = cache_entry->crack.XFX.SPR = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 21872 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_from_Special_Purpose_Register_SPR;

    /* semantic routine */
#line 3389 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move from Special Purpose Register\n"));
#line 21882 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_from_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3389 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        /* HACK - time base registers need to be calculated */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_dec:
            *rT = cpu_get_decrementer(processor);
            break;
      	case spr_tbrl:
        	  if (is_64bit_implementation) *rT = TB;
        	  else                         *rT = EXTRACTED64(TB, 32, 63);
      	break;
      	case spr_tbru:
        	  if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
      	  else                         *rT = EXTRACTED64(TB, 0, 31);
      	break;
          case spr_tbu:
          case spr_tbl:
            /* NOTE - these SPR's are not readable. Use mftb[ul] */
          default:
            *rT = SPREG(n);
            break;
          }
        }
        else {
          *rT = SPREG(n);
        }
      }
      PPC_INSN_FROM_SPR(RT_BITMASK, n);
    }
#line 21929 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21932 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_Indexed(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_Indexed
#line 1844 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic Indexed\n"));
#line 21951 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.343,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.343,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21979 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_Indexed;

    /* semantic routine */
#line 1844 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic Indexed\n"));
#line 21989 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1844 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(signed, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 22008 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.343,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22012 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Invalidate_All(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Invalidate_All
#undef MY_PREFIX
#define MY_PREFIX TLB_Invalidate_All
#line 4996 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode TLB Invalidate All\n"));
#line 22031 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.370,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c0002e4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.370,31./ #define get-values-from-icache|put-values-in-icache */
#line 22040 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Invalidate_All;

    /* semantic routine */
#line 4996 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics TLB Invalidate All\n"));
#line 22050 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Invalidate_All, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4996 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        int nr = 0;
        cpu *proc;
        while (1) {
          proc = psim_cpu(cpu_system(processor), nr);
          if (proc == NULL) break;
          cpu_page_tlb_invalidate_all(proc);
          nr++;
        }
      }
    }
#line 22075 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.370,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22078 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Time_Base(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Time_Base
#undef MY_PREFIX
#define MY_PREFIX Move_From_Time_Base
#line 4839 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Time Base\n"));
#line 22097 "icache.c"

  /* validate: 0.31,6.RT,11.tbr,21.371,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002e6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.tbr,21.371,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 4839 "./ppc-instructions"
    unsigned const tbr UNUSED = cache_entry->crack.XFX.tbr = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 22114 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Time_Base;

    /* semantic routine */
#line 4839 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Time Base\n"));
#line 22124 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Time_Base, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4839 "./ppc-instructions"
    {
      int n = (tbr_5_9_ << 5) | tbr_0_4_;
      if (n == 268) {
        if (is_64bit_implementation) *rT = TB;
        else                         *rT = EXTRACTED64(TB, 32, 63);
      }
      else if (n == 269) {
        if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
        else                         *rT = EXTRACTED64(TB, 0, 31);
      }
      else
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
    }
#line 22148 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.tbr,21.371,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22151 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_with_Update_Indexed(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_with_Update_Indexed
#line 1871 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic with Update Indexed\n"));
#line 22170 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.375,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.375,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22198 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_with_Update_Indexed;

    /* semantic routine */
#line 1871 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic with Update Indexed\n"));
#line 22208 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1871 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(signed, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 22228 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.375,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22232 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_Indexed(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_Indexed
#line 2071 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word Indexed\n"));
#line 22251 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.407,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00032e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.407,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22279 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_Indexed;

    /* semantic routine */
#line 2071 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word Indexed\n"));
#line 22289 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2071 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 2, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 22308 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.407,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22312 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_with_Complement_Rc0(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX OR_with_Complement_Rc0
#line 3080 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR with Complement\n"));
#line 22331 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000338)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3080 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 22361 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_with_Complement_Rc0;

    /* semantic routine */
#line 3080 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR with Complement\n"));
#line 22371 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3080 "./ppc-instructions"
    {
      *rA = *rS | ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22387 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22391 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_with_Complement_Rc1(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX OR_with_Complement_Rc1
#line 3080 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR with Complement\n"));
#line 22410 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000338)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3080 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 22440 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_with_Complement_Rc1;

    /* semantic routine */
#line 3080 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR with Complement\n"));
#line 22450 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3080 "./ppc-instructions"
    {
      *rA = *rS | ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22466 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22470 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_with_Update_Indexed(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_with_Update_Indexed
#line 2098 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word with Update Indexed\n"));
#line 22489 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.439,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00036e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.439,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22517 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_with_Update_Indexed;

    /* semantic routine */
#line 2098 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word with Update Indexed\n"));
#line 22527 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2098 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 2, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 22547 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.439,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22551 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Rc0(cpu *processor,
              instruction_word instruction,
              unsigned_word cia,
              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR
#undef MY_PREFIX
#define MY_PREFIX OR_Rc0
#line 3020 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR\n"));
#line 22570 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000378)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3020 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 22600 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Rc0;

    /* semantic routine */
#line 3020 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR\n"));
#line 22610 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3020 "./ppc-instructions"
    {
      *rA = *rS | *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22626 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22630 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Rc1(cpu *processor,
              instruction_word instruction,
              unsigned_word cia,
              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR
#undef MY_PREFIX
#define MY_PREFIX OR_Rc1
#line 3020 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR\n"));
#line 22649 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000378)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3020 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 22679 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Rc1;

    /* semantic routine */
#line 3020 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR\n"));
#line 22689 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3020 "./ppc-instructions"
    {
      *rA = *rS | *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22705 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22709 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE0_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE0_Rc0
#line 2767 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 22728 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2767 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2767 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 22760 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE0_Rc0;

    /* semantic routine */
#line 2767 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 22770 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2767 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 22795 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22799 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE0_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE0_Rc1
#line 2767 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 22818 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2767 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2767 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 22850 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE0_Rc1;

    /* semantic routine */
#line 2767 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 22860 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2767 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 22885 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22889 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Special_Purpose_Register_SPR256(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_to_Special_Purpose_Register_SPR256
#line 3345 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Special Purpose Register\n"));
#line 22908 "icache.c"

  /* validate: 0.31,6.RS,11.SPR,21.467,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0003a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3345 "./ppc-instructions"
    unsigned const SPR UNUSED = 256; /* default in icache - constant field */
#line 22925 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Special_Purpose_Register_SPR256;

    /* semantic routine */
#line 3345 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Special Purpose Register\n"));
#line 22935 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3345 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n)
               || spr_is_readonly(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        spreg new_val = (spr_length(n) == 64
      		   ? *rS
      		   : MASKED(*rS, 32, 63));
        /* HACK - time base registers need to be updated immediatly */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_tbu:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 32, 63)
                               | INSERTED64(new_val, 0, 31)));
            break;
          case spr_tbl:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 0, 31)
                               | INSERTED64(new_val, 32, 63)));
            break;
          case spr_dec:
            cpu_set_decrementer(processor, new_val);
            break;
          default:
            SPREG(n) = new_val;
            break;
          }
        }
        else {
          SPREG(n) = new_val;
        }
      }
      PPC_INSN_TO_SPR(RS_BITMASK, n);
    }
#line 22985 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22988 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Special_Purpose_Register_SPR(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_to_Special_Purpose_Register_SPR
#line 3345 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Special Purpose Register\n"));
#line 23007 "icache.c"

  /* validate: 0.31,6.RS,11.SPR,21.467,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0003a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3345 "./ppc-instructions"
    unsigned const SPR UNUSED = cache_entry->crack.XFX.SPR = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 23024 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Special_Purpose_Register_SPR;

    /* semantic routine */
#line 3345 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Special Purpose Register\n"));
#line 23034 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3345 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n)
               || spr_is_readonly(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        spreg new_val = (spr_length(n) == 64
      		   ? *rS
      		   : MASKED(*rS, 32, 63));
        /* HACK - time base registers need to be updated immediatly */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_tbu:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 32, 63)
                               | INSERTED64(new_val, 0, 31)));
            break;
          case spr_tbl:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 0, 31)
                               | INSERTED64(new_val, 32, 63)));
            break;
          case spr_dec:
            cpu_set_decrementer(processor, new_val);
            break;
          default:
            SPREG(n) = new_val;
            break;
          }
        }
        else {
          SPREG(n) = new_val;
        }
      }
      PPC_INSN_TO_SPR(RS_BITMASK, n);
    }
#line 23084 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #undef get-values-from-icache|put-values-in-icache */
#line 23087 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Invalidate(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Invalidate
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Invalidate
#line 4917 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Invalidate\n"));
#line 23106 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.470,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0003ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.470,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 23128 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Invalidate;

    /* semantic routine */
#line 4917 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Invalidate\n"));
#line 23138 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Invalidate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4917 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        TRACE(trace_tbd,("Data Cache Block Invalidate\n"));
    }
#line 23155 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.470,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23159 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NAND_Rc0(cpu *processor,
                instruction_word instruction,
                unsigned_word cia,
                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NAND
#undef MY_PREFIX
#define MY_PREFIX NAND_Rc0
#line 3040 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NAND\n"));
#line 23178 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0003b8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3040 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23208 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NAND_Rc0;

    /* semantic routine */
#line 3040 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NAND\n"));
#line 23218 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3040 "./ppc-instructions"
    {
      *rA = ~(*rS & *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 23234 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23238 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NAND_Rc1(cpu *processor,
                instruction_word instruction,
                unsigned_word cia,
                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NAND
#undef MY_PREFIX
#define MY_PREFIX NAND_Rc1
#line 3040 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NAND\n"));
#line 23257 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0003b8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3040 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23287 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NAND_Rc1;

    /* semantic routine */
#line 3040 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NAND\n"));
#line 23297 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3040 "./ppc-instructions"
    {
      *rA = ~(*rS & *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 23313 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23317 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE0_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE0_Rc0
#line 2745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 23336 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2745 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23368 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE0_Rc0;

    /* semantic routine */
#line 2745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 23378 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2745 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23404 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23408 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE0_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE0_Rc1
#line 2745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 23427 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2745 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23459 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE0_Rc1;

    /* semantic routine */
#line 2745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 23469 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2745 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23495 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23499 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_from_XER(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_from_XER
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_from_XER
#line 3445 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Condition Register from XER\n"));
#line 23518 "icache.c"

  /* validate: 0.31,6.BF,9./,11./,16./,21.512,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7fffff) != 0x7c000400)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,11./,16./,21.512,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 23531 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_from_XER;

    /* semantic routine */
#line 3445 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Condition Register from XER\n"));
#line 23541 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_from_XER, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3445 "./ppc-instructions"
    error("%s:%d:0x%08lx:%s unimplemented\n",
          itable[MY_INDEX].file, itable[MY_INDEX].line_nr, (long)cia, itable[MY_INDEX].name);
#line 23553 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,11./,16./,21.512,31./ #undef get-values-from-icache|put-values-in-icache */
#line 23556 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE1_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE1_Rc0
#line 2596 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 23575 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2596 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2596 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23607 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE1_Rc0;

    /* semantic routine */
#line 2596 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 23617 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2596 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23636 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23640 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE1_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE1_Rc1
#line 2596 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 23659 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2596 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2596 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23691 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE1_Rc1;

    /* semantic routine */
#line 2596 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 23701 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2596 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23720 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23724 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE1_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE1_Rc0
#line 2586 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 23743 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2586 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2586 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23775 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE1_Rc0;

    /* semantic routine */
#line 2586 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 23785 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2586 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23801 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23805 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE1_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE1_Rc1
#line 2586 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 23824 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2586 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2586 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23856 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE1_Rc1;

    /* semantic routine */
#line 2586 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 23866 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2586 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23882 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23886 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_String_Word_Indexed(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_String_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_String_Word_Indexed
#line 2328 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load String Word Indexed\n"));
#line 23905 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.533,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.533,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 23933 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_String_Word_Indexed;

    /* semantic routine */
#line 2328 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load String Word Indexed\n"));
#line 23943 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_String_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2328 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int n;
      int r;
      int i;
      int nr;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      n = EXTRACTED32(XER, 25, 31);
      r = RT - 1;
      i = 32;
      nr = (n + 3) / 4;
      if (((RT + nr >= 32)
           ? ((RA >= RT || RA < (RT + nr) % 32)
              || (RB >= RT || RB < (RT + nr) % 32))
           : ((RA >= RT && RA < RT + nr)
              || (RB >= RT && RB < RT + nr)))
          || (RT == RA || RT == RB))
        program_interrupt(processor, cia,
                        illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) {
          r = (r + 1) % 32;
          GPR(r) = 0;
        }
        GPR(r) |= INSERTED(MEM(unsigned, EA, 1), i, i+7);
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 23989 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.533,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23993 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_ByteReverse_Indexed(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_ByteReverse_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_ByteReverse_Indexed
#line 2215 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word Byte-Reverse Indexed\n"));
#line 24012 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.534,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.534,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24040 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_ByteReverse_Indexed;

    /* semantic routine */
#line 2215 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word Byte-Reverse Indexed\n"));
#line 24050 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_ByteReverse_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2215 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = SWAP_4(MEM(unsigned, EA, 4));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24069 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.534,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24073 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_Indexed
#line 3478 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single Indexed\n"));
#line 24092 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.535,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.535,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24127 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_Indexed;

    /* semantic routine */
#line 3478 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single Indexed\n"));
#line 24137 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3478 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24156 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.535,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24161 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Word_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Word_Rc0
#line 3271 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Word\n"));
#line 24180 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000430)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3271 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 24210 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Word_Rc0;

    /* semantic routine */
#line 3271 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Word\n"));
#line 24220 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3271 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source >> n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, \
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 24245 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24249 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Word_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Word_Rc1
#line 3271 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Word\n"));
#line 24268 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000430)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3271 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 24298 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Word_Rc1;

    /* semantic routine */
#line 3271 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Word\n"));
#line 24308 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3271 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source >> n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, \
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 24333 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24337 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE1_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE1_Rc0
#line 2542 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 24356 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2542 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2542 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 24388 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE1_Rc0;

    /* semantic routine */
#line 2542 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 24398 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2542 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 24416 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24420 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE1_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE1_Rc1
#line 2542 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 24439 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2542 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2542 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 24471 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE1_Rc1;

    /* semantic routine */
#line 2542 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 24481 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2542 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 24499 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24503 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Synchronize(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Synchronize
#undef MY_PREFIX
#define MY_PREFIX TLB_Synchronize
#line 5011 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode TLB Synchronize\n"));
#line 24522 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.566,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c00046c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.566,31./ #define get-values-from-icache|put-values-in-icache */
#line 24531 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Synchronize;

    /* semantic routine */
#line 5011 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics TLB Synchronize\n"));
#line 24541 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 5011 "./ppc-instructions"
    {
      /* nothing happens here - always in sync */
    }
#line 24554 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.566,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24557 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_with_Update_Indexed(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_with_Update_Indexed
#line 3505 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single with Update Indexed\n"));
#line 24576 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.567,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00046e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.567,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24611 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_with_Update_Indexed;

    /* semantic routine */
#line 3505 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single with Update Indexed\n"));
#line 24621 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3505 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24641 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.567,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24646 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Segment_Register(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Segment_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Segment_Register
#line 4954 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Segment Register\n"));
#line 24665 "icache.c"

  /* validate: 0.31,6.RT,11./,12.SR,16./,21.595,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc10ffff) != 0x7c0004a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,12.SR,16./,21.595,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 4954 "./ppc-instructions"
    unsigned const SR UNUSED = cache_entry->crack.X.SR = EXTRACTED32(instruction, 12, 15); /* default in icache - extraction */
#line 24682 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Segment_Register;

    /* semantic routine */
#line 4954 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Segment Register\n"));
#line 24692 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Segment_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4954 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        *rT = SEGREG(SR);
    }
#line 24709 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,12.SR,16./,21.595,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24712 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_String_Word_Immediate(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_String_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Load_String_Word_Immediate
#line 2296 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load String Word Immediate\n"));
#line 24731 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.NB,21.597,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004aa)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.NB,21.597,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2296 "./ppc-instructions"
    unsigned const NB UNUSED = cache_entry->crack.X.NB = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 24755 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_String_Word_Immediate;

    /* semantic routine */
#line 2296 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load String Word Immediate\n"));
#line 24765 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_String_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2296 "./ppc-instructions"
    {
      unsigned_word EA;
      int n;
      int r;
      int i;
      int nr;
      if (RA_is_0) EA = 0;
      else         EA = *rA;
      if (NB == 0) n = 32;
      else         n = NB;
      r = RT - 1;
      i = 32;
      nr = (n + 3) / 4;
      if ((RT + nr >= 32)
          ? (RA >= RT || RA < (RT + nr) % 32)
          : (RA >= RT && RA < RT + nr))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) {
          r = (r + 1) % 32;
          GPR(r) = 0;
        }
        GPR(r) |= INSERTED(MEM(unsigned, EA, 1), i, i+7);
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 24807 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.NB,21.597,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24811 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Synchronize(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Synchronize
#undef MY_PREFIX
#define MY_PREFIX Synchronize
#line 2500 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Synchronize\n"));
#line 24830 "icache.c"

  /* validate: 0.31,6./,9.L,11./,16./,21.598,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xff9fffff) != 0x7c0004ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,9.L,11./,16./,21.598,31./ #define get-values-from-icache|put-values-in-icache */
#line 2500 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 9, 10); /* default in icache - extraction */
#line 24841 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Synchronize;

    /* semantic routine */
#line 2500 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Synchronize\n"));
#line 24851 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2500 "./ppc-instructions"
    {
      /* do nothing */
    }
#line 24864 "icache.c"
    return nia;
    /* extraction: 0.31,6./,9.L,11./,16./,21.598,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24867 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_Indexed
#line 3532 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double Indexed\n"));
#line 24886 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.599,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.599,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24921 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_Indexed;

    /* semantic routine */
#line 3532 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double Indexed\n"));
#line 24931 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3532 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *frT = MEM(unsigned, EA, 8);
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24950 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.599,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24955 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE1_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE1_Rc0
#line 2676 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 24974 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2676 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2676 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25000 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE1_Rc0;

    /* semantic routine */
#line 2676 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 25010 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2676 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 25027 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25031 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE1_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE1_Rc1
#line 2676 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 25050 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2676 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2676 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 25076 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE1_Rc1;

    /* semantic routine */
#line 2676 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 25086 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2676 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 25103 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25107 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_with_Update_Indexed(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_with_Update_Indexed
#line 3559 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double with Update Indexed\n"));
#line 25126 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.631,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.631,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25161 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_with_Update_Indexed;

    /* semantic routine */
#line 3559 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double with Update Indexed\n"));
#line 25171 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3559 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *frT = MEM(unsigned, EA, 8);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 25191 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.631,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 25196 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE1_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE1_Rc0
#line 2620 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 25215 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2620 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2620 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25247 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE1_Rc0;

    /* semantic routine */
#line 2620 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 25257 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2620 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25275 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25279 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE1_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE1_Rc1
#line 2620 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 25298 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2620 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2620 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 25330 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE1_Rc1;

    /* semantic routine */
#line 2620 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 25340 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2620 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25358 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25362 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE1_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE1_Rc0
#line 2609 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 25381 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2609 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2609 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25413 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE1_Rc0;

    /* semantic routine */
#line 2609 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 25423 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2609 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25440 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25444 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE1_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE1_Rc1
#line 2609 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 25463 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2609 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2609 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 25495 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE1_Rc1;

    /* semantic routine */
#line 2609 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 25505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2609 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25522 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25526 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Segment_Register_Indirect(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Segment_Register_Indirect
#undef MY_PREFIX
#define MY_PREFIX Move_From_Segment_Register_Indirect
#line 4965 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Segment Register Indirect\n"));
#line 25545 "icache.c"

  /* validate: 0.31,6.RT,11./,16.RB,21.659,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07ff) != 0x7c000526)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16.RB,21.659,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25566 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Segment_Register_Indirect;

    /* semantic routine */
#line 4965 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Segment Register Indirect\n"));
#line 25576 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Segment_Register_Indirect, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4965 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        *rT = SEGREG(EXTRACTED32(*rB, 0, 3));
    }
#line 25593 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16.RB,21.659,31./ #undef get-values-from-icache|put-values-in-icache */
#line 25596 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_String_Word_Indexed(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_String_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_String_Word_Indexed
#line 2386 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store String Word Indexed\n"));
#line 25615 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.661,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.661,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25643 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_String_Word_Indexed;

    /* semantic routine */
#line 2386 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store String Word Indexed\n"));
#line 25653 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_String_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2386 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int n;
      int r;
      int i;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      n = EXTRACTED32(XER, 25, 31);
      r = RS - 1;
      i = 32;
      while (n > 0) {
        if (i == 32) r = (r + 1) % 32;
        STORE(EA, 1, EXTRACTED(GPR(r), i, i+7));
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 25686 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.661,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25690 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_ByteReversed_Indexed(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_ByteReversed_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_ByteReversed_Indexed
#line 2241 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word Byte-Reversed Indexed\n"));
#line 25709 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.662,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.662,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25737 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_ByteReversed_Indexed;

    /* semantic routine */
#line 2241 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word Byte-Reversed Indexed\n"));
#line 25747 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_ByteReversed_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2241 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, SWAP_4(*rS));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 25766 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.662,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25770 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_Indexed(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_Indexed
#line 3591 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single Indexed\n"));
#line 25789 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.663,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.663,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25824 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_Indexed;

    /* semantic routine */
#line 3591 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single Indexed\n"));
#line 25834 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3591 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, SINGLE(*frS));
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 25853 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.663,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 25858 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_with_Update_Indexed(cpu *processor,
                                                      instruction_word instruction,
                                                      unsigned_word cia,
                                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_with_Update_Indexed
#line 3618 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single with Update Indexed\n"));
#line 25877 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.695,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00056e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.695,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25912 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_with_Update_Indexed;

    /* semantic routine */
#line 3618 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single with Update Indexed\n"));
#line 25922 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3618 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 4, SINGLE(*frS));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 25942 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.695,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 25947 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE1_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE1_Rc0
#line 2665 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 25966 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2665 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2665 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25992 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE1_Rc0;

    /* semantic routine */
#line 2665 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 26002 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2665 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26019 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26023 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE1_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE1_Rc1
#line 2665 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 26042 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2665 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2665 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26068 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE1_Rc1;

    /* semantic routine */
#line 2665 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 26078 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2665 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26095 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26099 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE1_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE1_Rc0
#line 2655 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 26118 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2655 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2655 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26144 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE1_Rc0;

    /* semantic routine */
#line 2655 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 26154 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2655 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26170 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26174 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE1_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE1_Rc1
#line 2655 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 26193 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2655 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2655 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26219 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE1_Rc1;

    /* semantic routine */
#line 2655 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 26229 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2655 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26245 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26249 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_String_Word_Immedate(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_String_Word_Immedate
#undef MY_PREFIX
#define MY_PREFIX Store_String_Word_Immedate
#line 2364 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store String Word Immedate\n"));
#line 26268 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.NB,21.725,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005aa)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.NB,21.725,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2364 "./ppc-instructions"
    unsigned const NB UNUSED = cache_entry->crack.X.NB = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 26292 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_String_Word_Immedate;

    /* semantic routine */
#line 2364 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store String Word Immedate\n"));
#line 26302 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_String_Word_Immedate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2364 "./ppc-instructions"
    {
      unsigned_word EA;
      int n;
      int r;
      int i;
      if (RA_is_0) EA = 0;
      else         EA = *rA;
      if (NB == 0) n = 32;
      else         n = NB;
      r = RS - 1;
      i = 32;
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) r = (r + 1) % 32;
        STORE(EA, 1, EXTRACTED(GPR(r), i, i+7));
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 26334 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.NB,21.725,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26338 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_Indexed(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_Indexed
#line 3645 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double Indexed\n"));
#line 26357 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.727,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.727,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 26392 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_Indexed;

    /* semantic routine */
#line 3645 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double Indexed\n"));
#line 26402 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3645 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 8, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 26421 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.727,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 26426 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE1_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE1_Rc0
#line 2643 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 26445 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2643 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2643 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26471 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE1_Rc0;

    /* semantic routine */
#line 2643 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 26481 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2643 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26499 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26503 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE1_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE1_Rc1
#line 2643 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 26522 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2643 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2643 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26548 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE1_Rc1;

    /* semantic routine */
#line 2643 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 26558 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2643 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26576 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26580 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE1_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE1_Rc0
#line 2632 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 26599 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2632 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2632 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26625 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE1_Rc0;

    /* semantic routine */
#line 2632 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 26635 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2632 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26652 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26656 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE1_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE1_Rc1
#line 2632 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 26675 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2632 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2632 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26701 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE1_Rc1;

    /* semantic routine */
#line 2632 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 26711 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2632 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26728 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26732 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE1_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE1_Rc0
#line 2698 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 26751 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2698 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2698 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26783 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE1_Rc0;

    /* semantic routine */
#line 2698 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 26793 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2698 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 26814 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26818 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE1_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE1_Rc1
#line 2698 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 26837 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2698 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2698 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26869 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE1_Rc1;

    /* semantic routine */
#line 2698 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 26879 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2698 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 26900 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26904 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_with_Update_Indexed(cpu *processor,
                                                      instruction_word instruction,
                                                      unsigned_word cia,
                                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_with_Update_Indexed
#line 3684 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double with Update Indexed\n"));
#line 26923 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.759,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.759,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 26958 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_with_Update_Indexed;

    /* semantic routine */
#line 3684 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double with Update Indexed\n"));
#line 26968 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3684 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 8, *frS);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 26988 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.759,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 26993 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE1_Rc0(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE1_Rc0
#line 2532 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 27012 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2532 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2532 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27044 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE1_Rc0;

    /* semantic routine */
#line 2532 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 27054 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2532 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 27070 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27074 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE1_Rc1(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE1_Rc1
#line 2532 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 27093 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2532 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2532 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27125 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE1_Rc1;

    /* semantic routine */
#line 2532 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 27135 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2532 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 27151 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27155 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_ByteReverse_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_ByteReverse_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_ByteReverse_Indexed
#line 2202 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Byte-Reverse Indexed\n"));
#line 27174 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.790,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00062c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.790,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 27202 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_ByteReverse_Indexed;

    /* semantic routine */
#line 2202 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Byte-Reverse Indexed\n"));
#line 27212 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_ByteReverse_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2202 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = SWAP_2(MEM(unsigned, EA, 2));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 27231 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.790,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27235 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Rc0
#line 3314 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word\n"));
#line 27254 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000630)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3314 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27284 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Rc0;

    /* semantic routine */
#line 3314 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word\n"));
#line 27294 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3314 "./ppc-instructions"
    {
      unsigned64 mask;
      int n = MASKED(*rB, 59, 63);
      signed32 source = (signed32)*rS; /* signed to keep sign bit */
      signed32 shifted = source >> n;
      int S = (MASKED(*rS,32,32) != 0);
      signed64 r = ((unsigned64) source);
      r = ((unsigned64) source) << 32 | (unsigned32) source;
      r = ROTL64(r,64-n);
      if (MASKED(*rB,58,58) == 0)
      	mask = (unsigned64) MASK64(n+32,63);
      else
      	mask = (unsigned64) 0;
      *rA = (signed_word) (r & mask | ((signed64) -1*S) & ~mask); /* if 64bit will sign extend */
      if (S && (MASKED(r & ~mask,32,63)!=0))
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27327 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27331 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Rc1
#line 3314 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word\n"));
#line 27350 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000630)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3314 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27380 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Rc1;

    /* semantic routine */
#line 3314 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word\n"));
#line 27390 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3314 "./ppc-instructions"
    {
      unsigned64 mask;
      int n = MASKED(*rB, 59, 63);
      signed32 source = (signed32)*rS; /* signed to keep sign bit */
      signed32 shifted = source >> n;
      int S = (MASKED(*rS,32,32) != 0);
      signed64 r = ((unsigned64) source);
      r = ((unsigned64) source) << 32 | (unsigned32) source;
      r = ROTL64(r,64-n);
      if (MASKED(*rB,58,58) == 0)
      	mask = (unsigned64) MASK64(n+32,63);
      else
      	mask = (unsigned64) 0;
      *rA = (signed_word) (r & mask | ((signed64) -1*S) & ~mask); /* if 64bit will sign extend */
      if (S && (MASKED(r & ~mask,32,63)!=0))
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27423 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27427 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Immediate_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Immediate_Rc0
#line 3292 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word Immediate\n"));
#line 27446 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000670)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3292 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.X.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3292 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27472 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Immediate_Rc0;

    /* semantic routine */
#line 3292 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word Immediate\n"));
#line 27482 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3292 "./ppc-instructions"
    {
      int n = SH;
      signed_word r = ROTL32(*rS, /*64*/32-n);
      signed_word m = MASK(n+32, 63);
      int S = MASKED(*rS, 32, 32);
      signed_word shifted = (r & m) | (S ? ~m : 0);
      *rA = shifted;
      if (S && ((r & ~m) & MASK(32, 63)) != 0)
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27508 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27512 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Immediate_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Immediate_Rc1
#line 3292 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word Immediate\n"));
#line 27531 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000670)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3292 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.X.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3292 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27557 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Immediate_Rc1;

    /* semantic routine */
#line 3292 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word Immediate\n"));
#line 27567 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3292 "./ppc-instructions"
    {
      int n = SH;
      signed_word r = ROTL32(*rS, /*64*/32-n);
      signed_word m = MASK(n+32, 63);
      int S = MASKED(*rS, 32, 32);
      signed_word shifted = (r & m) | (S ? ~m : 0);
      *rA = shifted;
      if (S && ((r & ~m) & MASK(32, 63)) != 0)
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27593 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27597 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Enforce_Inorder_Execution_of_IO(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Enforce_Inorder_Execution_of_IO
#undef MY_PREFIX
#define MY_PREFIX Enforce_Inorder_Execution_of_IO
#line 4828 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Enforce In-order Execution of I/O\n"));
#line 27616 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.854,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c0006ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.854,31./ #define get-values-from-icache|put-values-in-icache */
#line 27625 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Enforce_Inorder_Execution_of_IO;

    /* semantic routine */
#line 4828 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Enforce In-order Execution of I/O\n"));
#line 27635 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Enforce_Inorder_Execution_of_IO, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4828 "./ppc-instructions"
    {
      /* Since this model has no instruction overlap
         this instruction need do nothing */
    }
#line 27649 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.854,31./ #undef get-values-from-icache|put-values-in-icache */
#line 27652 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_ByteReversed_Indexed(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_ByteReversed_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_ByteReversed_Indexed
#line 2228 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word Byte-Reversed Indexed\n"));
#line 27671 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.918,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00072c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.918,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 27699 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_ByteReversed_Indexed;

    /* semantic routine */
#line 2228 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word Byte-Reversed Indexed\n"));
#line 27709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_ByteReversed_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2228 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 2, SWAP_2(*rS));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 27728 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.918,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27732 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Half_Word_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Half_Word_Rc0
#line 3100 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Half Word\n"));
#line 27751 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.922,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000734)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3100 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27775 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Half_Word_Rc0;

    /* semantic routine */
#line 3100 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Half Word\n"));
#line 27785 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3100 "./ppc-instructions"
    {
      *rA = (signed_word)(signed16)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27801 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27805 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Half_Word_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Half_Word_Rc1
#line 3100 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Half Word\n"));
#line 27824 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.922,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000734)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3100 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27848 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Half_Word_Rc1;

    /* semantic routine */
#line 3100 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Half Word\n"));
#line 27858 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3100 "./ppc-instructions"
    {
      *rA = (signed_word)(signed16)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27874 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27878 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Byte_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Byte
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Byte_Rc0
#line 3090 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Byte\n"));
#line 27897 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.954,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000774)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3090 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27921 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Byte_Rc0;

    /* semantic routine */
#line 3090 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Byte\n"));
#line 27931 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3090 "./ppc-instructions"
    {
      *rA = (signed_word)(signed8)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27947 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27951 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Byte_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Byte
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Byte_Rc1
#line 3090 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Byte\n"));
#line 27970 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.954,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000774)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3090 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27994 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Byte_Rc1;

    /* semantic routine */
#line 3090 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Byte\n"));
#line 28004 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3090 "./ppc-instructions"
    {
      *rA = (signed_word)(signed8)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 28020 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28024 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE1_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE1_Rc0
#line 2767 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 28043 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2767 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2767 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 28075 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE1_Rc0;

    /* semantic routine */
#line 2767 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 28085 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2767 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28110 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28114 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE1_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE1_Rc1
#line 2767 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 28133 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2767 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2767 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 28165 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE1_Rc1;

    /* semantic routine */
#line 2767 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 28175 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2767 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28200 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28204 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Instruction_Cache_Block_Invalidate(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Instruction_Cache_Block_Invalidate
#undef MY_PREFIX
#define MY_PREFIX Instruction_Cache_Block_Invalidate
#line 4764 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Instruction Cache Block Invalidate\n"));
#line 28223 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.982,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0007ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.982,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28245 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Instruction_Cache_Block_Invalidate;

    /* semantic routine */
#line 4764 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Instruction Cache Block Invalidate\n"));
#line 28255 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Instruction_Cache_Block_Invalidate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4764 "./ppc-instructions"
    {
      /* blindly flush all instruction cache entries */
#if WITH_IDECODE_CACHE_SIZE
      cpu_flush_icache(processor);
#endif
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 28272 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.982,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28276 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Integer_Word_Indexed(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Integer_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Integer_Word_Indexed
#line 3657 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Integer Word Indexed\n"));
#line 28295 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.983,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0007ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.983,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28330 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Integer_Word_Indexed;

    /* semantic routine */
#line 3657 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Integer Word Indexed\n"));
#line 28340 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Integer_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3657 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 28359 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.983,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 28364 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE1_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE1_Rc0
#line 2745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 28383 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2745 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 28415 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE1_Rc0;

    /* semantic routine */
#line 2745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 28425 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2745 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28451 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28455 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE1_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE1_Rc1
#line 2745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 28474 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2745 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 28506 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE1_Rc1;

    /* semantic routine */
#line 2745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 28516 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2745 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28542 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28546 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_set_to_Zero(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_set_to_Zero
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_set_to_Zero
#line 4804 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block set to Zero\n"));
#line 28565 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.1014,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0007ec)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.1014,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28587 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_set_to_Zero;

    /* semantic routine */
#line 4804 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block set to Zero\n"));
#line 28597 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_set_to_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4804 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block set to Zero\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 28611 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.1014,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28615 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero
#line 1885 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero\n"));
#line 28634 "icache.c"

  /* validate: 0.32,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x80000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.32,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28658 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero;

    /* semantic routine */
#line 1885 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero\n"));
#line 28668 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1885 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 4);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 28687 "icache.c"
    return nia;
    /* extraction: 0.32,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28691 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_with_Update(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_with_Update
#line 1911 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero with Update\n"));
#line 28710 "icache.c"

  /* validate: 0.33,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x84000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.33,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28734 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_with_Update;

    /* semantic routine */
#line 1911 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero with Update\n"));
#line 28744 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1911 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 4);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 28764 "icache.c"
    return nia;
    /* extraction: 0.33,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28768 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero
#line 1722 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero\n"));
#line 28787 "icache.c"

  /* validate: 0.34,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x88000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.34,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28811 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero;

    /* semantic routine */
#line 1722 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero\n"));
#line 28821 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1722 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 1);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 28840 "icache.c"
    return nia;
    /* extraction: 0.34,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28844 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_with_Update(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_with_Update
#line 1749 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero with Update\n"));
#line 28863 "icache.c"

  /* validate: 0.35,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x8c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.35,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28887 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_with_Update;

    /* semantic routine */
#line 1749 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero with Update\n"));
#line 28897 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1749 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 1);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 28917 "icache.c"
    return nia;
    /* extraction: 0.35,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28921 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word(cpu *processor,
                  instruction_word instruction,
                  unsigned_word cia,
                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Word
#line 2112 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word\n"));
#line 28940 "icache.c"

  /* validate: 0.36,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x90000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.36,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28964 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word;

    /* semantic routine */
#line 2112 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word\n"));
#line 28974 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2112 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 4, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 28993 "icache.c"
    return nia;
    /* extraction: 0.36,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28997 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_with_Update(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Word_with_Update
#line 2138 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word with Update\n"));
#line 29016 "icache.c"

  /* validate: 0.37,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x94000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.37,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29040 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_with_Update;

    /* semantic routine */
#line 2138 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word with Update\n"));
#line 29050 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2138 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 4, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29070 "icache.c"
    return nia;
    /* extraction: 0.37,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29074 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte(cpu *processor,
                  instruction_word instruction,
                  unsigned_word cia,
                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte
#undef MY_PREFIX
#define MY_PREFIX Store_Byte
#line 2004 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte\n"));
#line 29093 "icache.c"

  /* validate: 0.38,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x98000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.38,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29117 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte;

    /* semantic routine */
#line 2004 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte\n"));
#line 29127 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2004 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 1, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 29146 "icache.c"
    return nia;
    /* extraction: 0.38,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29150 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_with_Update(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_with_Update
#line 2030 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte with Update\n"));
#line 29169 "icache.c"

  /* validate: 0.39,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x9c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.39,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29193 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_with_Update;

    /* semantic routine */
#line 2030 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte with Update\n"));
#line 29203 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2030 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 1, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29223 "icache.c"
    return nia;
    /* extraction: 0.39,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29227 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero
#line 1777 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero\n"));
#line 29246 "icache.c"

  /* validate: 0.40,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.40,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29270 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero;

    /* semantic routine */
#line 1777 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero\n"));
#line 29280 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1777 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29299 "icache.c"
    return nia;
    /* extraction: 0.40,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29303 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_with_Update(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_with_Update
#line 1803 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero with Update\n"));
#line 29322 "icache.c"

  /* validate: 0.41,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.41,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29346 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_with_Update;

    /* semantic routine */
#line 1803 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero with Update\n"));
#line 29356 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1803 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 29376 "icache.c"
    return nia;
    /* extraction: 0.41,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29380 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic
#line 1831 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic\n"));
#line 29399 "icache.c"

  /* validate: 0.42,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.42,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29423 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic;

    /* semantic routine */
#line 1831 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic\n"));
#line 29433 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1831 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(signed, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29452 "icache.c"
    return nia;
    /* extraction: 0.42,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29456 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_with_Update(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_with_Update
#line 1857 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic with Update\n"));
#line 29475 "icache.c"

  /* validate: 0.43,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xac000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.43,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29499 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_with_Update;

    /* semantic routine */
#line 1857 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic with Update\n"));
#line 29509 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1857 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(signed, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 29529 "icache.c"
    return nia;
    /* extraction: 0.43,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29533 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word
#line 2058 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word\n"));
#line 29552 "icache.c"

  /* validate: 0.44,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.44,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29576 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word;

    /* semantic routine */
#line 2058 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word\n"));
#line 29586 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2058 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 2, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 29605 "icache.c"
    return nia;
    /* extraction: 0.44,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29609 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_with_Update(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_with_Update
#line 2084 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word with Update\n"));
#line 29628 "icache.c"

  /* validate: 0.45,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.45,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29652 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_with_Update;

    /* semantic routine */
#line 2084 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word with Update\n"));
#line 29662 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2084 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 2, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29682 "icache.c"
    return nia;
    /* extraction: 0.45,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29686 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Multiple_Word(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Multiple_Word
#undef MY_PREFIX
#define MY_PREFIX Load_Multiple_Word
#line 2255 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Multiple Word\n"));
#line 29705 "icache.c"

  /* validate: 0.46,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.46,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29729 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Multiple_Word;

    /* semantic routine */
#line 2255 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Multiple Word\n"));
#line 29739 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Multiple_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2255 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int r;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      r = RT;
      if (RA >= r)
        program_interrupt(processor, cia,
                        illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT || (EA % 4 != 0))
        alignment_interrupt(processor, cia, EA);
      while (r <= 31) {
        GPR(r) = MEM(unsigned, EA, 4);
        r = r + 1;
        EA = EA + 4;
      }
    }
#line 29768 "icache.c"
    return nia;
    /* extraction: 0.46,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29772 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Multiple_Word(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Multiple_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Multiple_Word
#line 2274 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Multiple Word\n"));
#line 29791 "icache.c"

  /* validate: 0.47,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xbc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.47,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29815 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Multiple_Word;

    /* semantic routine */
#line 2274 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Multiple Word\n"));
#line 29825 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Multiple_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2274 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int r;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT
          || (EA % 4 != 0))
        alignment_interrupt(processor, cia, EA);
      r = RS;
      while (r <= 31) {
        STORE(EA, 4, GPR(r));
        r = r + 1;
        EA = EA + 4;
      }
    }
#line 29852 "icache.c"
    return nia;
    /* extraction: 0.47,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29856 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single
#line 3465 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single\n"));
#line 29875 "icache.c"

  /* validate: 0.48,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.48,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29906 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single;

    /* semantic routine */
#line 3465 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single\n"));
#line 29916 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3465 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29935 "icache.c"
    return nia;
    /* extraction: 0.48,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 29940 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_with_Update(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_with_Update
#line 3491 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single with Update\n"));
#line 29959 "icache.c"

  /* validate: 0.49,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.49,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29990 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_with_Update;

    /* semantic routine */
#line 3491 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single with Update\n"));
#line 30000 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3491 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30020 "icache.c"
    return nia;
    /* extraction: 0.49,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double
#line 3519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double\n"));
#line 30044 "icache.c"

  /* validate: 0.50,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.50,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30075 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double;

    /* semantic routine */
#line 3519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double\n"));
#line 30085 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3519 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *frT = MEM(unsigned, EA, 8);
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30104 "icache.c"
    return nia;
    /* extraction: 0.50,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30109 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_with_Update(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_with_Update
#line 3545 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double with Update\n"));
#line 30128 "icache.c"

  /* validate: 0.51,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xcc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.51,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30159 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_with_Update;

    /* semantic routine */
#line 3545 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double with Update\n"));
#line 30169 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3545 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *frT = MEM(unsigned, EA, 8);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30189 "icache.c"
    return nia;
    /* extraction: 0.51,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30194 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single
#line 3578 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single\n"));
#line 30213 "icache.c"

  /* validate: 0.52,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.52,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30244 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single;

    /* semantic routine */
#line 3578 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single\n"));
#line 30254 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3578 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 4, SINGLE(*frS));
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30273 "icache.c"
    return nia;
    /* extraction: 0.52,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30278 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_with_Update(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_with_Update
#line 3604 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single with Update\n"));
#line 30297 "icache.c"

  /* validate: 0.53,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.53,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30328 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_with_Update;

    /* semantic routine */
#line 3604 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single with Update\n"));
#line 30338 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3604 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 4, SINGLE(*frS));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30358 "icache.c"
    return nia;
    /* extraction: 0.53,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30363 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double
#line 3632 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double\n"));
#line 30382 "icache.c"

  /* validate: 0.54,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.54,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30413 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double;

    /* semantic routine */
#line 3632 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double\n"));
#line 30423 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3632 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 8, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30442 "icache.c"
    return nia;
    /* extraction: 0.54,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30447 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_with_Update(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_with_Update
#line 3670 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double with Update\n"));
#line 30466 "icache.c"

  /* validate: 0.55,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xdc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.55,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30497 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_with_Update;

    /* semantic routine */
#line 3670 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double with Update\n"));
#line 30507 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3670 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 8, *frS);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30527 "icache.c"
    return nia;
    /* extraction: 0.55,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30532 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Single_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Single_Rc0
#line 3933 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide Single\n"));
#line 30551 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3933 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 30585 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Single_Rc0;

    /* semantic routine */
#line 3933 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide Single\n"));
#line 30595 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3933 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       1 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       1 /*single?*/);
      }
      else {
        /*HACK!*/
        float s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30634 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30640 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Single_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Single_Rc1
#line 3933 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide Single\n"));
#line 30659 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3933 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 30693 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Single_Rc1;

    /* semantic routine */
#line 3933 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide Single\n"));
#line 30703 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3933 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       1 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       1 /*single?*/);
      }
      else {
        /*HACK!*/
        float s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30742 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30748 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Single_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Single_Rc0
#line 3822 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract Single\n"));
#line 30767 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3822 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 30801 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Single_Rc0;

    /* semantic routine */
#line 3822 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract Single\n"));
#line 30811 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3822 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30843 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30849 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Single_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Single_Rc1
#line 3822 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract Single\n"));
#line 30868 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3822 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 30902 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Single_Rc1;

    /* semantic routine */
#line 3822 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract Single\n"));
#line 30912 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3822 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30944 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30950 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Single_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Single_Rc0
#line 3770 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add Single\n"));
#line 30969 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3770 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31003 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Single_Rc0;

    /* semantic routine */
#line 3770 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add Single\n"));
#line 31013 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3770 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 31045 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 31051 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Single_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Single_Rc1
#line 3770 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add Single\n"));
#line 31070 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3770 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31104 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Single_Rc1;

    /* semantic routine */
#line 3770 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add Single\n"));
#line 31114 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3770 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 31146 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 31152 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Single_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Single_Rc0
#line 4727 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root Single\n"));
#line 31171 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4727 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31200 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Single_Rc0;

    /* semantic routine */
#line 4727 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root Single\n"));
#line 31210 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4727 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31223 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31228 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Single_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Single_Rc1
#line 4727 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root Single\n"));
#line 31247 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4727 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31276 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Single_Rc1;

    /* semantic routine */
#line 4727 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root Single\n"));
#line 31286 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4727 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31299 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31304 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Estimate_Single_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Estimate_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Estimate_Single_Rc0
#line 4730 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Estimate Single\n"));
#line 31323 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4730 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31352 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Estimate_Single_Rc0;

    /* semantic routine */
#line 4730 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Estimate Single\n"));
#line 31362 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Estimate_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4730 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31375 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31380 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Estimate_Single_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Estimate_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Estimate_Single_Rc1
#line 4730 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Estimate Single\n"));
#line 31399 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4730 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31428 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Estimate_Single_Rc1;

    /* semantic routine */
#line 4730 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Estimate Single\n"));
#line 31438 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Estimate_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4730 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31451 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31456 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_Single_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_Single_Rc0
#line 3874 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply Single\n"));
#line 31475 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xec000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3874 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31509 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_Single_Rc0;

    /* semantic routine */
#line 3874 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply Single\n"));
#line 31519 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3874 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 31551 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 31557 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_Single_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_Single_Rc1
#line 3874 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply Single\n"));
#line 31576 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xec000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3874 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31610 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_Single_Rc1;

    /* semantic routine */
#line 3874 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply Single\n"));
#line 31620 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3874 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 31652 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 31658 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_Single_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_Single_Rc0
#line 4107 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract Single\n"));
#line 31677 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4107 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31716 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_Single_Rc0;

    /* semantic routine */
#line 4107 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract Single\n"));
#line 31726 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4107 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product - *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 31779 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 31786 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_Single_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_Single_Rc1
#line 4107 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract Single\n"));
#line 31805 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4107 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31844 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_Single_Rc1;

    /* semantic routine */
#line 4107 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract Single\n"));
#line 31854 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4107 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product - *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 31907 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 31914 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_Single_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_Single_Rc0
#line 4013 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add Single\n"));
#line 31933 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4013 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31972 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_Single_Rc0;

    /* semantic routine */
#line 4013 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add Single\n"));
#line 31982 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4013 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product + *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32035 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32042 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_Single_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_Single_Rc1
#line 4013 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add Single\n"));
#line 32061 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4013 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 32100 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_Single_Rc1;

    /* semantic routine */
#line 4013 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add Single\n"));
#line 32110 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4013 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product + *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32163 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32170 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_Single_Rc0(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_Single_Rc0
#line 4295 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract Single\n"));
#line 32189 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4295 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 32228 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_Single_Rc0;

    /* semantic routine */
#line 4295 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract Single\n"));
#line 32238 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4295 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product - *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32291 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32298 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_Single_Rc1(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_Single_Rc1
#line 4295 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract Single\n"));
#line 32317 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4295 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 32356 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_Single_Rc1;

    /* semantic routine */
#line 4295 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract Single\n"));
#line 32366 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4295 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product - *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32419 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32426 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_Single_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_Single_Rc0
#line 4201 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add Single\n"));
#line 32445 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4201 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 32484 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_Single_Rc0;

    /* semantic routine */
#line 4201 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add Single\n"));
#line 32494 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4201 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product + *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32547 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32554 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_Single_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_Single_Rc1
#line 4201 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add Single\n"));
#line 32573 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4201 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 32612 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_Single_Rc1;

    /* semantic routine */
#line 4201 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add Single\n"));
#line 32622 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4201 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product + *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32675 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32682 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Compare_Unordered(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Compare_Unordered
#undef MY_PREFIX
#define MY_PREFIX Floating_Compare_Unordered
#line 4622 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Compare Unordered\n"));
#line 32701 "icache.c"

  /* validate: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc6007ff) != 0xfc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.X.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.X.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 32732 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Compare_Unordered;

    /* semantic routine */
#line 4622 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Compare Unordered\n"));
#line 32742 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Compare_Unordered, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4622 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned c;
      if (is_NaN(*frA, 0) || is_NaN(*frB, 0))
        c = cr_i_summary_overflow; /* 0b0001 - (FRA) ? (FRB) */
      else if (is_less_than(frA, frB))
        c = cr_i_negative; /* 0b1000 - (FRA) < (FRB) */
      else if (is_greater_than(frA, frB))
        c = cr_i_positive; /* 0b0100 - (FRA) > (FRB) */
      else
        c = cr_i_zero; /* 0b0010 - (FRA) = (FRB) */
      FPSCR_SET_FPCC(c);
      CR_SET(BF, c); /* CR[4*BF..4*BF+3] = c */
      if (is_SNaN(*frA, 0) || is_SNaN(*frB, 0))
        FPSCR_OR_VX(fpscr_vxsnan);
      FPSCR_END(0);
      PPC_INSN_FLOAT_CR(0, FRA_BITMASK | FRB_BITMASK, BF_BITMASK);
    }
#line 32770 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRA
#undef FRB
#line 32775 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Round_to_SinglePrecision_Rc0(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Round_to_SinglePrecision
#undef MY_PREFIX
#define MY_PREFIX Floating_Round_to_SinglePrecision_Rc0
#line 4347 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Round to Single-Precision\n"));
#line 32794 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000018)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4347 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 32823 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Round_to_SinglePrecision_Rc0;

    /* semantic routine */
#line 4347 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Round to Single-Precision\n"));
#line 32833 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Round_to_SinglePrecision, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4347 "./ppc-instructions"
    {
      int sign;
      int exp;
      unsigned64 frac_grx;
      /***/
        /* split off cases for what to do */
        if (EXTRACTED64(*frB, 1, 11) < 897
            && EXTRACTED64(*frB, 1, 63) > 0) {
            if ((FPSCR & fpscr_ue) == 0) GOTO(Disabled_Exponent_Underflow);
            if ((FPSCR & fpscr_ue) != 0) GOTO(Enabled_Exponent_Underflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 1150
            && EXTRACTED64(*frB, 1, 11) < 2047) {
            if ((FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
            if ((FPSCR & fpscr_oe) != 0) GOTO(Enabled_Exponent_Overflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 896
            && EXTRACTED64(*frB, 1, 11) < 1151) GOTO(Normal_Operand);
        if (EXTRACTED64(*frB, 1, 63) == 0) GOTO(Zero_Operand);
        if (EXTRACTED64(*frB, 1, 11) == 2047) {
          if (EXTRACTED64(*frB, 12, 63) == 0) GOTO(Infinity_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 1) GOTO(QNaN_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 0
              && EXTRACTED64(*frB, 13, 63) > 0) GOTO(SNaN_Operand);
        }
      /**/
      LABEL(Disabled_Exponent_Underflow):
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
          /* G|R|X == zero from above */
          while (exp < -126) {
            exp = exp + 1;
            frac_grx = (INSERTED64(EXTRACTED64(frac_grx, 0, 54), 1, 55)
                        | MASKED64(frac_grx, 55, 55));
          }
        FPSCR_SET_UX(EXTRACTED64(frac_grx, 24, 55) > 0);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (EXTRACTED64(frac_grx, 0, 52) == 0) {
          *frT = INSERTED64(sign, 0, 0);
          if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
          if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        }
        if (EXTRACTED64(frac_grx, 0, 52) > 0) {
          if (EXTRACTED64(frac_grx, 0, 0) == 1) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
          if (EXTRACTED64(frac_grx, 0, 0) == 0) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_denormalized_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_denormalized_number);
          }
          /*Normalize_Operand:*/
            while (EXTRACTED64(frac_grx, 0, 0) == 0) {
              exp = exp - 1;
              frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1,  52), 0, 51);
            }
          *frT = (INSERTED64(sign, 0, 0)
                  | INSERTED64(exp + 1023, 1, 11)
                  | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        }
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Underflow):
        FPSCR_SET_UX(1);
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = (BIT64(0) |
                      INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52));
        }
        /*Normalize_Operand:*/
          while (EXTRACTED64(frac_grx, 0, 0) == 0) {
            exp = exp - 1;
            frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1, 52), 0, 51);
          }
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        exp = exp + 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Disabled_Exponent_Overflow):
        FPSCR_SET_OX(1);
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_to_nearest) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_zero) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_pos_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_neg_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        /* FPSCR[FR] <- undefined */
        FPSCR_SET_FI(1);
        FPSCR_SET_XX(1);
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Overflow):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
      /**/
      LABEL(Enabled_Overflow):
        FPSCR_SET_OX(1);
        exp = exp - 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Zero_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Infinity_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(QNaN_Operand):
        *frT = INSERTED64(EXTRACTED64(*frB, 0, 34), 0, 34);
        FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(SNaN_Operand):
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0) {
          *frT = (MASKED64(*frB, 0, 11)
                  | BIT64(12)
                  | MASKED64(*frB, 13, 34));
          FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        }
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Normal_Operand):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (exp > 127 && (FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
        if (exp > 127 && (FPSCR & fpscr_oe) != 0) GOTO(Enabled_Overflow);
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Done):
        PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33057 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33062 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Round_to_SinglePrecision_Rc1(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Round_to_SinglePrecision
#undef MY_PREFIX
#define MY_PREFIX Floating_Round_to_SinglePrecision_Rc1
#line 4347 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Round to Single-Precision\n"));
#line 33081 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000018)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4347 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33110 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Round_to_SinglePrecision_Rc1;

    /* semantic routine */
#line 4347 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Round to Single-Precision\n"));
#line 33120 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Round_to_SinglePrecision, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4347 "./ppc-instructions"
    {
      int sign;
      int exp;
      unsigned64 frac_grx;
      /***/
        /* split off cases for what to do */
        if (EXTRACTED64(*frB, 1, 11) < 897
            && EXTRACTED64(*frB, 1, 63) > 0) {
            if ((FPSCR & fpscr_ue) == 0) GOTO(Disabled_Exponent_Underflow);
            if ((FPSCR & fpscr_ue) != 0) GOTO(Enabled_Exponent_Underflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 1150
            && EXTRACTED64(*frB, 1, 11) < 2047) {
            if ((FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
            if ((FPSCR & fpscr_oe) != 0) GOTO(Enabled_Exponent_Overflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 896
            && EXTRACTED64(*frB, 1, 11) < 1151) GOTO(Normal_Operand);
        if (EXTRACTED64(*frB, 1, 63) == 0) GOTO(Zero_Operand);
        if (EXTRACTED64(*frB, 1, 11) == 2047) {
          if (EXTRACTED64(*frB, 12, 63) == 0) GOTO(Infinity_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 1) GOTO(QNaN_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 0
              && EXTRACTED64(*frB, 13, 63) > 0) GOTO(SNaN_Operand);
        }
      /**/
      LABEL(Disabled_Exponent_Underflow):
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
          /* G|R|X == zero from above */
          while (exp < -126) {
            exp = exp + 1;
            frac_grx = (INSERTED64(EXTRACTED64(frac_grx, 0, 54), 1, 55)
                        | MASKED64(frac_grx, 55, 55));
          }
        FPSCR_SET_UX(EXTRACTED64(frac_grx, 24, 55) > 0);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (EXTRACTED64(frac_grx, 0, 52) == 0) {
          *frT = INSERTED64(sign, 0, 0);
          if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
          if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        }
        if (EXTRACTED64(frac_grx, 0, 52) > 0) {
          if (EXTRACTED64(frac_grx, 0, 0) == 1) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
          if (EXTRACTED64(frac_grx, 0, 0) == 0) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_denormalized_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_denormalized_number);
          }
          /*Normalize_Operand:*/
            while (EXTRACTED64(frac_grx, 0, 0) == 0) {
              exp = exp - 1;
              frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1,  52), 0, 51);
            }
          *frT = (INSERTED64(sign, 0, 0)
                  | INSERTED64(exp + 1023, 1, 11)
                  | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        }
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Underflow):
        FPSCR_SET_UX(1);
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = (BIT64(0) |
                      INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52));
        }
        /*Normalize_Operand:*/
          while (EXTRACTED64(frac_grx, 0, 0) == 0) {
            exp = exp - 1;
            frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1, 52), 0, 51);
          }
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        exp = exp + 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Disabled_Exponent_Overflow):
        FPSCR_SET_OX(1);
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_to_nearest) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_zero) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_pos_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_neg_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        /* FPSCR[FR] <- undefined */
        FPSCR_SET_FI(1);
        FPSCR_SET_XX(1);
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Overflow):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
      /**/
      LABEL(Enabled_Overflow):
        FPSCR_SET_OX(1);
        exp = exp - 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Zero_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Infinity_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(QNaN_Operand):
        *frT = INSERTED64(EXTRACTED64(*frB, 0, 34), 0, 34);
        FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(SNaN_Operand):
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0) {
          *frT = (MASKED64(*frB, 0, 11)
                  | BIT64(12)
                  | MASKED64(*frB, 13, 34));
          FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        }
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Normal_Operand):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (exp > 127 && (FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
        if (exp > 127 && (FPSCR & fpscr_oe) != 0) GOTO(Enabled_Overflow);
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Done):
        PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33344 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33349 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_Rc0
#line 4568 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word\n"));
#line 33368 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4568 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33397 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_Rc0;

    /* semantic routine */
#line 4568 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word\n"));
#line 33407 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4568 "./ppc-instructions"
    {
      floating_point_assist_interrupt(processor, cia);
    }
#line 33420 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33425 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_Rc1
#line 4568 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word\n"));
#line 33444 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4568 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33473 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_Rc1;

    /* semantic routine */
#line 4568 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word\n"));
#line 33483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4568 "./ppc-instructions"
    {
      floating_point_assist_interrupt(processor, cia);
    }
#line 33496 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33501 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0(cpu *processor,
                                                                    instruction_word instruction,
                                                                    unsigned_word cia,
                                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0
#line 4575 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word with round towards Zero\n"));
#line 33520 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4575 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33549 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0;

    /* semantic routine */
#line 4575 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word with round towards Zero\n"));
#line 33559 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4575 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      convert_to_integer(processor, cia,
                         frT, *frB,
                         fpscr_rn_round_towards_zero, 32);
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33577 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33582 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1(cpu *processor,
                                                                    instruction_word instruction,
                                                                    unsigned_word cia,
                                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1
#line 4575 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word with round towards Zero\n"));
#line 33601 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4575 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33630 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1;

    /* semantic routine */
#line 4575 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word with round towards Zero\n"));
#line 33640 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4575 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      convert_to_integer(processor, cia,
                         frT, *frB,
                         fpscr_rn_round_towards_zero, 32);
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33658 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33663 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Rc0
#line 3900 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide\n"));
#line 33682 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3900 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33716 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Rc0;

    /* semantic routine */
#line 3900 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide\n"));
#line 33726 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3900 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       0 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       0 /*single?*/);
      }
      else {
        /*HACK!*/
        double s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33765 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33771 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Rc1
#line 3900 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide\n"));
#line 33790 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3900 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33824 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Rc1;

    /* semantic routine */
#line 3900 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide\n"));
#line 33834 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3900 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       0 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       0 /*single?*/);
      }
      else {
        /*HACK!*/
        double s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33873 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33879 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Rc0
#line 3796 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract\n"));
#line 33898 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3796 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33932 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Rc0;

    /* semantic routine */
#line 3796 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract\n"));
#line 33942 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3796 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33974 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33980 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Rc1
#line 3796 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract\n"));
#line 33999 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3796 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34033 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Rc1;

    /* semantic routine */
#line 3796 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract\n"));
#line 34043 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3796 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34075 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34081 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Rc0(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Rc0
#line 3744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add\n"));
#line 34100 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34134 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Rc0;

    /* semantic routine */
#line 3744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add\n"));
#line 34144 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3744 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34176 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34182 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Rc1(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Rc1
#line 3744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add\n"));
#line 34201 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34235 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Rc1;

    /* semantic routine */
#line 3744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add\n"));
#line 34245 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3744 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34277 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34283 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Rc0
#line 4724 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root\n"));
#line 34302 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4724 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34331 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Rc0;

    /* semantic routine */
#line 4724 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root\n"));
#line 34341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4724 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34354 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34359 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Rc1
#line 4724 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root\n"));
#line 34378 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4724 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34407 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Rc1;

    /* semantic routine */
#line 4724 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root\n"));
#line 34417 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4724 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34430 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34435 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC0_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC0_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 34454 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34493 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC0_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 34503 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 34525 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 34532 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC0_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC0_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 34551 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34590 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC0_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 34600 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 34622 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 34629 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC0_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC0_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 34648 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34682 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC0_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 34692 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 34724 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 34730 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC0_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC0_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 34749 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34783 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC0_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 34793 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 34825 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 34831 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Square_Root_Estimate_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Square_Root_Estimate
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Square_Root_Estimate_Rc0
#line 4733 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Square Root Estimate\n"));
#line 34850 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4733 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34879 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Square_Root_Estimate_Rc0;

    /* semantic routine */
#line 4733 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Square Root Estimate\n"));
#line 34889 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Square_Root_Estimate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4733 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34902 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34907 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Square_Root_Estimate_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Square_Root_Estimate
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Square_Root_Estimate_Rc1
#line 4733 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Square Root Estimate\n"));
#line 34926 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4733 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34955 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Square_Root_Estimate_Rc1;

    /* semantic routine */
#line 4733 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Square Root Estimate\n"));
#line 34965 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Square_Root_Estimate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4733 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34978 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34983 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC0_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC0_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 35002 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35041 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC0_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 35051 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35104 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35111 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC0_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC0_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 35130 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35169 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC0_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 35179 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35232 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35239 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC0_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC0_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 35258 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35297 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC0_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 35307 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35360 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35367 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC0_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC0_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 35386 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35425 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC0_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 35435 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35488 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35495 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC0_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC0_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 35514 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35553 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC0_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 35563 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35616 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35623 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC0_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC0_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 35642 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35681 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC0_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 35691 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35744 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35751 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC0_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC0_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 35770 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35809 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC0_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 35819 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35872 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35879 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC0_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC0_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 35898 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35937 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC0_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 35947 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 36000 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36007 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Compare_Ordered(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Compare_Ordered
#undef MY_PREFIX
#define MY_PREFIX Floating_Compare_Ordered
#line 4644 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Compare Ordered\n"));
#line 36026 "icache.c"

  /* validate: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc6007ff) != 0xfc000040)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.X.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.X.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 36057 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Compare_Ordered;

    /* semantic routine */
#line 4644 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Compare Ordered\n"));
#line 36067 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Compare_Ordered, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4644 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned c;
      if (is_NaN(*frA, 0) || is_NaN(*frB, 0))
        c = cr_i_summary_overflow; /* 0b0001 - (FRA) ? (FRB) */
      else if (is_less_than(frA, frB))
        c = cr_i_negative; /* 0b1000 - (FRA) < (FRB) */
      else if (is_greater_than(frA, frB))
        c = cr_i_positive; /* 0b0100 - (FRA) > (FRB) */
      else
        c = cr_i_zero; /* 0b0010 - (FRA) = (FRB) */
      FPSCR_SET_FPCC(c);
      CR_SET(BF, c); /* CR[4*BF..4*BF+3] = c */
      if (is_SNaN(*frA, 0) || is_SNaN(*frB, 0)) {
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0)
          FPSCR_OR_VX(fpscr_vxvc);
      }
      else if (is_QNaN(*frA, 0) || is_QNaN(*frB, 0)) {
        FPSCR_OR_VX(fpscr_vxvc);
      }
      FPSCR_END(0);
      PPC_INSN_FLOAT_CR(0, FRA_BITMASK | FRB_BITMASK, BF_BITMASK);
    }
#line 36101 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRA
#undef FRB
#line 36106 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_1_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_1
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_1_Rc0
#line 4707 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 1\n"));
#line 36125 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.38,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00004c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4707 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36148 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_1_Rc0;

    /* semantic routine */
#line 4707 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 1\n"));
#line 36158 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_1, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4707 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      if (bit & fpscr_fi)
        bit |= fpscr_xx;
      if ((bit & fpscr_vx_bits))
        bit |= fpscr_fx;
      /* note - omit vx bit */
      if ((bit & (fpscr_ox | fpscr_ux | fpscr_zx | fpscr_xx)))
        bit |= fpscr_fx;
      FPSCR |= bit;
      FPSCR_END(Rc);
    }
#line 36181 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 36184 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_1_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_1
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_1_Rc1
#line 4707 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 1\n"));
#line 36203 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.38,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00004c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4707 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36226 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_1_Rc1;

    /* semantic routine */
#line 4707 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 1\n"));
#line 36236 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_1, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4707 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      if (bit & fpscr_fi)
        bit |= fpscr_xx;
      if ((bit & fpscr_vx_bits))
        bit |= fpscr_fx;
      /* note - omit vx bit */
      if ((bit & (fpscr_ox | fpscr_ux | fpscr_zx | fpscr_xx)))
        bit |= fpscr_fx;
      FPSCR |= bit;
      FPSCR_END(Rc);
    }
#line 36259 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 36262 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negate_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negate
#undef MY_PREFIX
#define MY_PREFIX Floating_Negate_Rc0
#line 3712 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negate\n"));
#line 36281 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3712 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36310 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negate_Rc0;

    /* semantic routine */
#line 3712 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negate\n"));
#line 36320 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3712 "./ppc-instructions"
    {
      *frT = *frB ^ BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 36335 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 36340 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negate_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negate
#undef MY_PREFIX
#define MY_PREFIX Floating_Negate_Rc1
#line 3712 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negate\n"));
#line 36359 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3712 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36388 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negate_Rc1;

    /* semantic routine */
#line 3712 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negate\n"));
#line 36398 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3712 "./ppc-instructions"
    {
      *frT = *frB ^ BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 36413 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 36418 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC1_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC1_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 36437 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36476 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC1_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 36486 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 36508 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36515 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC1_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC1_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 36534 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36573 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC1_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 36583 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 36605 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36612 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC1_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC1_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 36631 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36665 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC1_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 36675 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 36707 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 36713 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC1_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC1_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 36732 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36766 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC1_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 36776 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 36808 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 36814 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC1_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC1_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 36833 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36872 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC1_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 36882 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 36935 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36942 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC1_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC1_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 36961 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37000 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC1_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 37010 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37063 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37070 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC1_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC1_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 37089 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37128 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC1_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 37138 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37191 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37198 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC1_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC1_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 37217 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37256 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC1_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 37266 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37319 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37326 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC1_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC1_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 37345 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37384 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC1_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 37394 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37447 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37454 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC1_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC1_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 37473 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37512 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC1_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 37522 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37575 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37582 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC1_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC1_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 37601 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37640 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC1_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 37650 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37703 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37710 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC1_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC1_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 37729 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37768 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC1_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 37778 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37831 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37838 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_from_FPSCR(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_from_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_from_FPSCR
#line 4678 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Condition Register from FPSCR\n"));
#line 37857 "icache.c"

  /* validate: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc63ffff) != 0xfc000080)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4678 "./ppc-instructions"
    unsigned const BFA UNUSED = cache_entry->crack.X.BFA = EXTRACTED32(instruction, 11, 13); /* default in icache - extraction */
#line 37880 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_from_FPSCR;

    /* semantic routine */
#line 4678 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Condition Register from FPSCR\n"));
#line 37890 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_from_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4678 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned field = FPSCR_FIELD(BFA);
      CR_SET(BF, field);
      FPSCR_SET(BFA, 0); /* FPSCR_END fixes up FEX/VX */
      FPSCR_END(0);
    }
#line 37907 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ #undef get-values-from-icache|put-values-in-icache */
#line 37910 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_0_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_0
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_0_Rc0
#line 4701 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 0\n"));
#line 37929 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.70,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00008c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4701 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37952 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_0_Rc0;

    /* semantic routine */
#line 4701 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 0\n"));
#line 37962 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_0, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4701 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      FPSCR &= ~bit;
      FPSCR_END(Rc);
    }
#line 37978 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 37981 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_0_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_0
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_0_Rc1
#line 4701 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 0\n"));
#line 38000 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.70,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00008c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 75 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4701 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38023 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_0_Rc1;

    /* semantic routine */
#line 4701 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 0\n"));
#line 38033 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_0, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4701 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      FPSCR &= ~bit;
      FPSCR_END(Rc);
    }
#line 38049 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 38052 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Move_Register_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Move_Register
#undef MY_PREFIX
#define MY_PREFIX Floating_Move_Register_Rc0
#line 3703 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Move Register\n"));
#line 38071 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000090)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3703 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38100 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Move_Register_Rc0;

    /* semantic routine */
#line 3703 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Move Register\n"));
#line 38110 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Move_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3703 "./ppc-instructions"
    {
      *frT = *frB;
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 38125 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 38130 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Move_Register_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Move_Register
#undef MY_PREFIX
#define MY_PREFIX Floating_Move_Register_Rc1
#line 3703 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Move Register\n"));
#line 38149 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000090)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3703 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38178 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Move_Register_Rc1;

    /* semantic routine */
#line 3703 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Move Register\n"));
#line 38188 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Move_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3703 "./ppc-instructions"
    {
      *frT = *frB;
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 38203 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 38208 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC2_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC2_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 38227 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38266 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC2_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 38276 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 38298 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38305 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC2_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC2_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 38324 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38363 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC2_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 38373 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 38395 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38402 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC2_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC2_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 38421 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38455 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC2_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 38465 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 38497 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 38503 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC2_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC2_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 38522 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38556 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC2_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 38566 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 38598 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 38604 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC2_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC2_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 38623 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38662 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC2_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 38672 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38725 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38732 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC2_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC2_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 38751 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38790 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC2_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 38800 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38853 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38860 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC2_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC2_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 38879 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38918 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC2_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 38928 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38981 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38988 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC2_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC2_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 39007 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39046 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC2_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 39056 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39109 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39116 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC2_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC2_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 39135 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39174 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC2_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 39184 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39237 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39244 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC2_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC2_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 39263 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39302 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC2_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 39312 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39365 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39372 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC2_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC2_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 39391 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39430 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC2_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 39440 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39493 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39500 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC2_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC2_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 39519 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39558 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC2_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 39568 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39621 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39628 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC3_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC3_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 39647 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39686 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC3_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 39696 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 39718 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39725 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC3_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC3_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 39744 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39783 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC3_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 39793 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 39815 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39822 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC3_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC3_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 39841 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39875 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC3_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 39885 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 39917 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 39923 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC3_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC3_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 39942 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39976 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC3_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 39986 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 40018 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 40024 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC3_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC3_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 40043 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40082 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC3_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 40092 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40145 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40152 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC3_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC3_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 40171 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40210 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC3_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 40220 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40273 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40280 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC3_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC3_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 40299 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40338 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC3_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 40348 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40401 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40408 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC3_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC3_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 40427 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40466 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC3_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 40476 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40529 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40536 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC3_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC3_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 40555 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40594 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC3_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 40604 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40657 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40664 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC3_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC3_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 40683 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40722 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC3_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 40732 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40785 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40792 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC3_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC3_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 40811 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40850 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC3_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 40860 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40913 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40920 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC3_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC3_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 40939 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40978 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC3_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 40988 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41041 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41048 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Field_Immediate_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Field_Immediate
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Field_Immediate_Rc0
#line 4685 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Field Immediate\n"));
#line 41067 "icache.c"

  /* validate: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7f0ffe) != 0xfc00010c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4685 "./ppc-instructions"
    unsigned const U UNUSED = cache_entry->crack.X.U = EXTRACTED32(instruction, 16, 19); /* default in icache - extraction */
#line 4685 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41092 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Field_Immediate_Rc0;

    /* semantic routine */
#line 4685 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Field Immediate\n"));
#line 41102 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Field_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4685 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      FPSCR_SET(BF, U);
      FPSCR_END(Rc);
    }
#line 41117 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 41120 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Field_Immediate_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Field_Immediate
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Field_Immediate_Rc1
#line 4685 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Field Immediate\n"));
#line 41139 "icache.c"

  /* validate: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7f0ffe) != 0xfc00010c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 67 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4685 "./ppc-instructions"
    unsigned const U UNUSED = cache_entry->crack.X.U = EXTRACTED32(instruction, 16, 19); /* default in icache - extraction */
#line 4685 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41164 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Field_Immediate_Rc1;

    /* semantic routine */
#line 4685 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Field Immediate\n"));
#line 41174 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Field_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4685 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      FPSCR_SET(BF, U);
      FPSCR_END(Rc);
    }
#line 41189 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 41192 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_Absolute_Value_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_Absolute_Value_Rc0
#line 3730 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Absolute Value\n"));
#line 41211 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3730 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41240 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_Absolute_Value_Rc0;

    /* semantic routine */
#line 3730 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Absolute Value\n"));
#line 41250 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3730 "./ppc-instructions"
    {
      *frT = *frB | BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 41265 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 41270 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_Absolute_Value_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_Absolute_Value_Rc1
#line 3730 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Absolute Value\n"));
#line 41289 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3730 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41318 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_Absolute_Value_Rc1;

    /* semantic routine */
#line 3730 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Absolute Value\n"));
#line 41328 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3730 "./ppc-instructions"
    {
      *frT = *frB | BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 41343 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 41348 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC4_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC4_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 41367 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41406 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC4_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 41416 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 41438 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41445 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC4_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC4_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 41464 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41503 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC4_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 41513 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 41535 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41542 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC4_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC4_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 41561 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41595 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC4_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 41605 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 41637 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 41643 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC4_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC4_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 41662 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41696 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC4_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 41706 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 41738 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 41744 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC4_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC4_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 41763 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41802 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC4_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 41812 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41865 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41872 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC4_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC4_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 41891 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41930 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC4_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 41940 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41993 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42000 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC4_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC4_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 42019 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42058 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC4_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 42068 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42121 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42128 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC4_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC4_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 42147 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42186 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC4_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 42196 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42249 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42256 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC4_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC4_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 42275 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42314 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC4_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 42324 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42377 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42384 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC4_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC4_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 42403 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42442 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC4_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 42452 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42505 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42512 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC4_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC4_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 42531 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42570 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC4_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 42580 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42633 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42640 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC4_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC4_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 42659 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42698 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC4_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 42708 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42761 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42768 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC5_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC5_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 42787 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42826 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC5_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 42836 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 42858 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42865 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC5_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC5_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 42884 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42923 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC5_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 42933 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 42955 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42962 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC5_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC5_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 42981 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43015 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC5_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 43025 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 43057 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 43063 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC5_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC5_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 43082 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43116 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC5_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 43126 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 43158 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 43164 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC5_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC5_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 43183 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43222 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC5_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 43232 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43285 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43292 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC5_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC5_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 43311 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43350 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC5_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 43360 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43413 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43420 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC5_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC5_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 43439 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43478 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC5_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 43488 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43541 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43548 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC5_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC5_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 43567 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43606 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC5_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 43616 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43669 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43676 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC5_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC5_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 43695 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43734 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC5_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 43744 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43797 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43804 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC5_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC5_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 43823 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43862 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC5_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 43872 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43925 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43932 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC5_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC5_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 43951 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43990 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC5_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 44000 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44053 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44060 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC5_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC5_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 44079 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44118 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC5_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 44128 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44181 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44188 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC6_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC6_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 44207 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44246 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC6_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 44256 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 44278 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44285 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC6_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC6_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 44304 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44343 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC6_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 44353 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 44375 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44382 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC6_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC6_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 44401 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44435 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC6_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 44445 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 44477 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 44483 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC6_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC6_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 44502 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44536 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC6_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 44546 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 44578 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 44584 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC6_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC6_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 44603 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44642 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC6_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 44652 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44705 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44712 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC6_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC6_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 44731 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44770 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC6_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 44780 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44833 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44840 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC6_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC6_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 44859 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44898 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC6_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 44908 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44961 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44968 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC6_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC6_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 44987 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45026 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC6_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 45036 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45089 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45096 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC6_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC6_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 45115 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45154 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC6_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 45164 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45217 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45224 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC6_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC6_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 45243 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45282 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC6_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 45292 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45345 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45352 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC6_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC6_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 45371 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45410 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC6_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 45420 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45473 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45480 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC6_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC6_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 45499 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45538 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC6_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 45548 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45601 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45608 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC7_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC7_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 45627 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45666 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC7_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 45676 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 45698 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45705 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC7_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC7_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 45724 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45763 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC7_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 45773 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 45795 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45802 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC7_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC7_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 45821 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45855 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC7_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 45865 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 45897 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 45903 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC7_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC7_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 45922 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45956 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC7_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 45966 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 45998 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 46004 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC7_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC7_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 46023 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46062 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC7_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 46072 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46125 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46132 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC7_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC7_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 46151 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46190 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC7_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 46200 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46253 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46260 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC7_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC7_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 46279 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46318 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC7_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 46328 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46381 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46388 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC7_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC7_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 46407 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46446 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC7_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 46456 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46509 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46516 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC7_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC7_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 46535 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46574 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC7_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 46584 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46637 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46644 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC7_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC7_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 46663 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46702 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC7_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 46712 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46765 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46772 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC7_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC7_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 46791 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46830 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC7_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 46840 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46893 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46900 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC7_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC7_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 46919 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46958 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC7_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 46968 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47021 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47028 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Absolute_Value_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Absolute_Value_Rc0
#line 3721 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Absolute Value\n"));
#line 47047 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000210)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3721 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47076 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Absolute_Value_Rc0;

    /* semantic routine */
#line 3721 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Absolute Value\n"));
#line 47086 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3721 "./ppc-instructions"
    {
      *frT = *frB & ~BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 47101 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 47106 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Absolute_Value_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Absolute_Value_Rc1
#line 3721 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Absolute Value\n"));
#line 47125 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000210)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3721 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47154 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Absolute_Value_Rc1;

    /* semantic routine */
#line 3721 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Absolute Value\n"));
#line 47164 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3721 "./ppc-instructions"
    {
      *frT = *frB & ~BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 47179 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 47184 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC8_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC8_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 47203 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47242 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC8_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 47252 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 47274 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47281 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC8_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC8_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 47300 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47339 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC8_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 47349 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 47371 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47378 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC8_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC8_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 47397 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47431 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC8_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 47441 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 47473 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 47479 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC8_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC8_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 47498 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47532 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC8_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 47542 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 47574 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 47580 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC8_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC8_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 47599 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47638 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC8_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 47648 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47701 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47708 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC8_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC8_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 47727 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47766 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC8_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 47776 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47829 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47836 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC8_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC8_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 47855 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47894 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC8_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 47904 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47957 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47964 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC8_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC8_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 47983 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48022 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC8_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 48032 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48085 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48092 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC8_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC8_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 48111 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48150 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC8_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 48160 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48213 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48220 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC8_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC8_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 48239 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48278 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC8_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 48288 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48341 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48348 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC8_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC8_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 48367 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48406 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC8_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 48416 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48469 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48476 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC8_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC8_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 48495 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48534 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC8_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 48544 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48597 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48604 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC9_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC9_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 48623 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48662 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC9_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 48672 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 48694 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48701 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC9_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC9_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 48720 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48759 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC9_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 48769 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 48791 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48798 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC9_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC9_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 48817 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48851 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC9_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 48861 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 48893 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 48899 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC9_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC9_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 48918 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48952 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC9_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 48962 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 48994 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 49000 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC9_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC9_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 49019 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49058 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC9_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 49068 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49121 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49128 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC9_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC9_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 49147 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49186 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC9_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 49196 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49249 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49256 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC9_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC9_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 49275 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49314 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC9_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 49324 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49377 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49384 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC9_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC9_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 49403 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49442 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC9_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 49452 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49505 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49512 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC9_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC9_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 49531 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49570 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC9_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 49580 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49633 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49640 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC9_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC9_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 49659 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49698 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC9_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 49708 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49761 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49768 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC9_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC9_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 49787 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49826 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC9_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 49836 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49889 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49896 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC9_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC9_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 49915 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49954 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC9_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 49964 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50017 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50024 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC10_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC10_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 50043 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50082 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC10_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 50092 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 50114 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50121 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC10_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC10_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 50140 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50179 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC10_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 50189 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 50211 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50218 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC10_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC10_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 50237 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50271 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC10_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 50281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 50313 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 50319 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC10_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC10_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 50338 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50372 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC10_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 50382 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 50414 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 50420 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC10_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC10_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 50439 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50478 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC10_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 50488 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50541 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50548 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC10_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC10_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 50567 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50606 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC10_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 50616 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50669 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50676 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC10_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC10_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 50695 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50734 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC10_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 50744 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50797 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50804 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC10_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC10_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 50823 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50862 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC10_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 50872 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50925 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50932 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC10_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC10_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 50951 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50990 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC10_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 51000 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51053 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51060 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC10_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC10_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 51079 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51118 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC10_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 51128 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51181 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51188 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC10_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC10_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 51207 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51246 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC10_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 51256 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51309 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51316 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC10_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC10_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 51335 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51374 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC10_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 51384 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51437 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51444 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC11_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC11_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 51463 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51502 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC11_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 51512 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 51534 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51541 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC11_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC11_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 51560 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51599 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC11_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 51609 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 51631 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51638 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC11_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC11_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 51657 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51691 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC11_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 51701 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 51733 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 51739 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC11_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC11_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 51758 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51792 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC11_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 51802 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 51834 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 51840 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC11_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC11_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 51859 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51898 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC11_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 51908 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51961 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51968 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC11_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC11_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 51987 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52026 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC11_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 52036 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52089 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52096 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC11_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC11_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 52115 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52154 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC11_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 52164 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52217 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52224 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC11_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC11_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 52243 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52282 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC11_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 52292 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52345 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52352 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC11_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC11_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 52371 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52410 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC11_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 52420 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52473 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52480 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC11_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC11_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 52499 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52538 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC11_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 52548 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52601 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52608 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC11_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC11_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 52627 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52666 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC11_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 52676 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52729 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52736 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC11_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC11_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 52755 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52794 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC11_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 52804 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52857 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52864 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC12_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC12_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 52883 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52922 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC12_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 52932 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 52954 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52961 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC12_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC12_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 52980 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53019 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC12_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 53029 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 53051 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53058 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC12_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC12_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 53077 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53111 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC12_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 53121 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 53153 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 53159 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC12_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC12_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 53178 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53212 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC12_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 53222 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 53254 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 53260 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC12_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC12_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 53279 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53318 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC12_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 53328 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53381 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53388 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC12_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC12_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 53407 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53446 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC12_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 53456 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53509 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53516 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC12_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC12_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 53535 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53574 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC12_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 53584 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53637 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53644 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC12_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC12_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 53663 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53702 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC12_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 53712 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53765 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53772 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC12_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC12_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 53791 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53830 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC12_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 53840 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53893 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53900 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC12_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC12_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 53919 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53958 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC12_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 53968 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54021 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54028 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC12_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC12_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 54047 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54086 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC12_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 54096 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54149 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54156 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC12_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC12_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 54175 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54214 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC12_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 54224 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54277 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54284 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC13_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC13_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 54303 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54342 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC13_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 54352 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 54374 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC13_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC13_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 54400 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54439 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC13_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 54449 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 54471 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54478 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC13_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC13_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 54497 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54531 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC13_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 54541 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 54573 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 54579 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC13_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC13_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 54598 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54632 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC13_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 54642 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 54674 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 54680 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC13_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC13_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 54699 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54738 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC13_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 54748 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54801 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54808 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC13_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC13_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 54827 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54866 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC13_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 54876 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54929 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54936 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC13_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC13_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 54955 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54994 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC13_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 55004 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55057 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55064 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC13_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC13_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 55083 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55122 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC13_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 55132 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55185 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55192 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC13_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC13_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 55211 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55250 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC13_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 55260 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55313 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55320 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC13_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC13_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 55339 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55378 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC13_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 55388 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55441 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55448 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC13_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC13_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 55467 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55506 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC13_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 55516 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55569 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55576 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC13_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC13_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 55595 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55634 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC13_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 55644 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55697 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55704 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC14_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC14_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 55723 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55762 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC14_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 55772 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 55794 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55801 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC14_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC14_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 55820 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55859 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC14_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 55869 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 55891 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55898 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC14_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC14_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 55917 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55951 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC14_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 55961 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 55993 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 55999 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC14_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC14_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 56018 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56052 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC14_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 56062 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 56094 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 56100 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC14_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC14_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 56119 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56158 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC14_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 56168 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56221 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56228 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC14_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC14_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 56247 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56286 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC14_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 56296 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56349 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56356 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC14_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC14_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 56375 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56414 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC14_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 56424 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56477 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56484 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC14_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC14_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 56503 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56542 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC14_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 56552 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56605 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56612 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC14_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC14_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 56631 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56670 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC14_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 56680 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56733 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56740 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC14_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC14_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 56759 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56798 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC14_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 56808 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56861 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56868 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC14_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC14_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 56887 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56926 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC14_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 56936 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56989 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56996 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC14_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC14_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 57015 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57054 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC14_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 57064 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57117 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57124 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC15_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC15_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 57143 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57182 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC15_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 57192 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 57214 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57221 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC15_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC15_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 57240 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57279 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC15_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 57289 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 57311 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57318 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC15_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC15_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 57337 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57371 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC15_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 57381 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 57413 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 57419 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC15_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC15_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 57438 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57472 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC15_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 57482 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 57514 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 57520 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC15_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC15_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 57539 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57578 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC15_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 57588 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57641 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57648 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC15_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC15_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 57667 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57706 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC15_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 57716 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57769 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57776 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC15_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC15_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 57795 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57834 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC15_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 57844 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57897 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57904 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC15_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC15_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 57923 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57962 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC15_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 57972 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58025 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58032 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC15_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC15_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 58051 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58090 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC15_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 58100 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58153 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58160 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC15_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC15_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 58179 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58218 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC15_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 58228 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58281 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58288 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC15_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC15_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 58307 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58346 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC15_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 58356 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58409 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58416 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC15_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC15_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 58435 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58474 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC15_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 58484 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58537 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58544 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC16_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC16_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 58563 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58602 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC16_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 58612 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 58634 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58641 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC16_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC16_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 58660 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58699 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC16_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 58709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 58731 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58738 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC16_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC16_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 58757 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58791 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC16_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 58801 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 58833 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 58839 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC16_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC16_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 58858 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58892 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC16_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 58902 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 58934 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 58940 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC16_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC16_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 58959 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58998 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC16_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 59008 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59061 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59068 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC16_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC16_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 59087 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59126 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC16_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 59136 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59189 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59196 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC16_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC16_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 59215 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59254 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC16_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 59264 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59317 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59324 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC16_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC16_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 59343 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59382 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC16_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 59392 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59445 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59452 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC16_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC16_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 59471 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59510 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC16_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 59520 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59573 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59580 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC16_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC16_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 59599 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59638 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC16_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 59648 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59701 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59708 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC16_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC16_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 59727 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59766 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC16_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 59776 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59829 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59836 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC16_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC16_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 59855 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59894 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC16_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 59904 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59957 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59964 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC17_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC17_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 59983 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60022 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC17_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 60032 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 60054 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60061 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC17_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC17_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 60080 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60119 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC17_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 60129 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 60151 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60158 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC17_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC17_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 60177 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60211 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC17_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 60221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 60253 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 60259 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC17_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC17_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 60278 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60312 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC17_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 60322 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 60354 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 60360 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC17_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC17_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 60379 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60418 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC17_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 60428 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60481 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60488 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC17_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC17_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 60507 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60546 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC17_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 60556 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60609 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60616 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC17_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC17_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 60635 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60674 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC17_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 60684 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60737 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60744 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC17_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC17_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 60763 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60802 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC17_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 60812 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60865 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60872 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC17_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC17_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 60891 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60930 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC17_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 60940 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60993 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61000 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC17_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC17_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 61019 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61058 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC17_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 61068 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61121 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61128 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC17_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC17_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 61147 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61186 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC17_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 61196 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61249 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61256 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC17_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC17_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 61275 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61314 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC17_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 61324 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61377 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61384 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_FPSCR_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_From_FPSCR_Rc0
#line 4673 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From FPSCR\n"));
#line 61403 "icache.c"

  /* validate: 0.63,6.FRT,11./,16./,21.583,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00048e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 4673 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61427 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_FPSCR_Rc0;

    /* semantic routine */
#line 4673 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From FPSCR\n"));
#line 61437 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4673 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      *frT = FPSCR;
      FPSCR_END(Rc);
    }
#line 61452 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#line 61456 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_FPSCR_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_From_FPSCR_Rc1
#line 4673 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From FPSCR\n"));
#line 61475 "icache.c"

  /* validate: 0.63,6.FRT,11./,16./,21.583,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00048e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 4673 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61499 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_FPSCR_Rc1;

    /* semantic routine */
#line 4673 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From FPSCR\n"));
#line 61509 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4673 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      *frT = FPSCR;
      FPSCR_END(Rc);
    }
#line 61524 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#line 61528 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC18_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC18_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 61547 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61586 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC18_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 61596 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 61618 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61625 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC18_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC18_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 61644 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61683 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC18_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 61693 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 61715 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61722 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC18_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC18_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 61741 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61775 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC18_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 61785 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 61817 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 61823 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC18_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC18_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 61842 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61876 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC18_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 61886 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 61918 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 61924 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC18_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC18_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 61943 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61982 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC18_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 61992 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62045 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62052 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC18_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC18_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 62071 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62110 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC18_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 62120 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62173 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62180 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC18_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC18_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 62199 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62238 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC18_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 62248 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62301 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62308 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC18_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC18_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 62327 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62366 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC18_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 62376 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62429 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62436 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC18_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC18_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 62455 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62494 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC18_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 62504 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62557 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62564 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC18_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC18_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 62583 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62622 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC18_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 62632 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62685 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62692 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC18_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC18_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 62711 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62750 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC18_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 62760 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62813 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62820 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC18_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC18_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 62839 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62878 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC18_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 62888 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62941 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62948 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC19_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC19_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 62967 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63006 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC19_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 63016 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 63038 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63045 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC19_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC19_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 63064 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63103 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC19_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 63113 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 63135 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63142 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC19_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC19_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 63161 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63195 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC19_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 63205 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 63237 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 63243 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC19_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC19_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 63262 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63296 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC19_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 63306 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 63338 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 63344 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC19_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC19_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 63363 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63402 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC19_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 63412 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63465 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63472 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC19_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC19_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 63491 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63530 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC19_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 63540 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63593 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63600 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC19_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC19_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 63619 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63658 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC19_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 63668 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63721 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63728 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC19_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC19_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 63747 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63786 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC19_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 63796 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63849 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63856 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC19_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC19_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 63875 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63914 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC19_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 63924 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63977 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63984 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC19_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC19_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 64003 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64042 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC19_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 64052 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64105 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64112 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC19_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC19_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 64131 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64170 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC19_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 64180 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64233 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64240 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC19_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC19_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 64259 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64298 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC19_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 64308 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64361 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64368 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC20_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC20_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 64387 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64426 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC20_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 64436 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 64458 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64465 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC20_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC20_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 64484 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64523 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC20_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 64533 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 64555 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64562 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC20_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC20_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 64581 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64615 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC20_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 64625 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 64657 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 64663 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC20_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC20_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 64682 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64716 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC20_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 64726 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 64758 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 64764 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC20_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC20_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 64783 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64822 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC20_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 64832 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64885 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64892 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC20_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC20_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 64911 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64950 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC20_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 64960 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65013 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65020 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC20_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC20_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 65039 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65078 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC20_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 65088 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65141 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65148 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC20_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC20_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 65167 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65206 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC20_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 65216 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65269 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65276 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC20_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC20_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 65295 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65334 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC20_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 65344 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65397 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65404 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC20_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC20_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 65423 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65462 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC20_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 65472 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65525 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65532 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC20_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC20_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 65551 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65590 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC20_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 65600 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65653 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65660 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC20_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC20_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 65679 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65718 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC20_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 65728 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65781 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65788 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC21_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC21_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 65807 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65846 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC21_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 65856 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 65878 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65885 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC21_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC21_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 65904 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65943 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC21_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 65953 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 65975 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65982 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC21_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC21_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 66001 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66035 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC21_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 66045 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 66077 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 66083 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC21_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC21_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 66102 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66136 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC21_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 66146 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 66178 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 66184 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC21_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC21_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 66203 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66242 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC21_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 66252 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66305 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66312 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC21_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC21_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 66331 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66370 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC21_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 66380 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66433 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66440 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC21_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC21_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 66459 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66498 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC21_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 66508 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66561 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66568 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC21_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC21_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 66587 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66626 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC21_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 66636 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66689 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66696 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC21_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC21_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 66715 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66754 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC21_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 66764 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66817 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66824 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC21_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC21_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 66843 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66882 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC21_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 66892 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66945 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66952 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC21_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC21_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 66971 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67010 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC21_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 67020 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67073 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67080 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC21_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC21_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 67099 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67138 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC21_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 67148 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67201 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67208 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Fields_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Fields_Rc0
#line 4690 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Fields\n"));
#line 67227 "icache.c"

  /* validate: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfe0107fe) != 0xfc00058e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 4690 "./ppc-instructions"
    unsigned const FLM UNUSED = cache_entry->crack.XFL.FLM = EXTRACTED32(instruction, 7, 14); /* default in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.XFL.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.XFL.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4690 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67253 "icache.c"

#define insn cache_entry->crack.XFL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Fields_Rc0;

    /* semantic routine */
#line 4690 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Fields\n"));
#line 67263 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4690 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      int i;
      for (i = 0; i < 8; i++) {
        if ((FLM & BIT8(i))) {
          FPSCR &= ~MASK32(i*4, i*4+3);
          FPSCR |= MASKED32(*frB, i*4, i*4+3);
        }
      }
      FPSCR_END(Rc);
    }
#line 67284 "icache.c"
    return nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRB
#line 67288 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Fields_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Fields_Rc1
#line 4690 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Fields\n"));
#line 67307 "icache.c"

  /* validate: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfe0107fe) != 0xfc00058e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 4690 "./ppc-instructions"
    unsigned const FLM UNUSED = cache_entry->crack.XFL.FLM = EXTRACTED32(instruction, 7, 14); /* default in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.XFL.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.XFL.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4690 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67333 "icache.c"

#define insn cache_entry->crack.XFL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Fields_Rc1;

    /* semantic routine */
#line 4690 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Fields\n"));
#line 67343 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4690 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      int i;
      for (i = 0; i < 8; i++) {
        if ((FLM & BIT8(i))) {
          FPSCR &= ~MASK32(i*4, i*4+3);
          FPSCR |= MASKED32(*frB, i*4, i*4+3);
        }
      }
      FPSCR_END(Rc);
    }
#line 67364 "icache.c"
    return nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRB
#line 67368 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC22_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC22_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 67387 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67426 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC22_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 67436 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 67458 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67465 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC22_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC22_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 67484 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67523 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC22_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 67533 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 67555 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67562 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC22_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC22_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 67581 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67615 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC22_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 67625 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 67657 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 67663 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC22_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC22_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 67682 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67716 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC22_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 67726 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 67758 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 67764 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC22_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC22_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 67783 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67822 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC22_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 67832 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67885 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67892 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC22_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC22_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 67911 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67950 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC22_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 67960 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68013 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68020 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC22_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC22_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 68039 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68078 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC22_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 68088 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68141 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68148 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC22_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC22_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 68167 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68206 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC22_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 68216 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68269 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68276 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC22_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC22_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 68295 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68334 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC22_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 68344 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68397 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68404 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC22_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC22_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 68423 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68462 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC22_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 68472 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68525 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68532 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC22_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC22_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 68551 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68590 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC22_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 68600 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68653 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68660 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC22_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC22_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 68679 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68718 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC22_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 68728 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68781 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68788 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC23_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC23_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 68807 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68846 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC23_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 68856 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 68878 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68885 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC23_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC23_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 68904 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68943 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC23_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 68953 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 68975 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68982 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC23_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC23_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 69001 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69035 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC23_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 69045 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 69077 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 69083 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC23_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC23_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 69102 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69136 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC23_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 69146 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 69178 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 69184 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC23_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC23_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 69203 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69242 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC23_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 69252 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69305 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69312 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC23_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC23_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 69331 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69370 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC23_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 69380 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69433 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69440 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC23_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC23_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 69459 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69498 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC23_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 69508 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69561 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69568 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC23_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC23_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 69587 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69626 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC23_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 69636 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69689 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69696 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC23_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC23_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 69715 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69754 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC23_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 69764 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69817 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69824 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC23_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC23_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 69843 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69882 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC23_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 69892 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69945 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69952 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC23_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC23_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 69971 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70010 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC23_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 70020 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70073 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70080 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC23_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC23_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 70099 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70138 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC23_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 70148 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70201 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70208 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC24_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC24_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 70227 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70266 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC24_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 70276 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 70298 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70305 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC24_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC24_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 70324 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70363 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC24_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 70373 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 70395 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70402 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC24_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC24_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 70421 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70455 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC24_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 70465 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 70497 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 70503 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC24_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC24_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 70522 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70556 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC24_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 70566 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 70598 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 70604 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC24_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC24_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 70623 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70662 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC24_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 70672 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70725 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70732 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC24_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC24_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 70751 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70790 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC24_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 70800 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70853 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70860 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC24_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC24_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 70879 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70918 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC24_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 70928 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70981 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70988 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC24_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC24_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 71007 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71046 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC24_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 71056 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71109 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71116 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC24_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC24_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 71135 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71174 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC24_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 71184 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71237 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71244 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC24_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC24_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 71263 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71302 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC24_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 71312 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71365 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71372 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC24_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC24_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 71391 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71430 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC24_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 71440 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71493 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71500 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC24_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC24_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 71519 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71558 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC24_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 71568 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71621 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71628 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC25_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC25_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 71647 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71686 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC25_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 71696 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 71718 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71725 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC25_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC25_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 71744 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71783 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC25_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 71793 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 71815 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71822 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC25_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC25_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 71841 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71875 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC25_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 71885 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 71917 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 71923 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC25_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC25_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 71942 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71976 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC25_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 71986 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 72018 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 72024 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC25_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC25_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 72043 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72082 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC25_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 72092 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72145 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72152 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC25_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC25_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 72171 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72210 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC25_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 72220 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72273 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72280 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC25_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC25_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 72299 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72338 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC25_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 72348 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72401 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72408 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC25_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC25_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 72427 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72466 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC25_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 72476 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72529 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72536 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC25_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC25_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 72555 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72594 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC25_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 72604 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72657 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72664 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC25_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC25_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 72683 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72722 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC25_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 72732 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72785 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72792 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC25_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC25_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 72811 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72850 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC25_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 72860 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72913 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72920 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC25_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC25_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 72939 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72978 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC25_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 72988 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73041 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73048 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC26_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC26_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 73067 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73106 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC26_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 73116 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 73138 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73145 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC26_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC26_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 73164 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73203 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC26_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 73213 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 73235 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73242 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC26_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC26_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 73261 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73295 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC26_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 73305 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 73337 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 73343 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC26_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC26_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 73362 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73396 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC26_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 73406 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 73438 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 73444 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC26_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC26_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 73463 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73502 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC26_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 73512 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73565 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73572 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC26_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC26_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 73591 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73630 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC26_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 73640 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73693 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73700 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC26_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC26_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 73719 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73758 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC26_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 73768 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73821 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73828 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC26_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC26_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 73847 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73886 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC26_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 73896 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73949 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73956 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC26_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC26_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 73975 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74014 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC26_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 74024 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74077 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74084 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC26_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC26_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 74103 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74142 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC26_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 74152 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74205 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74212 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC26_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC26_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 74231 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74270 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC26_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 74280 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74333 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74340 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC26_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC26_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 74359 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74398 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC26_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 74408 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74461 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74468 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC27_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC27_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 74487 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74526 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC27_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 74536 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 74558 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74565 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC27_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC27_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 74584 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74623 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC27_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 74633 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 74655 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74662 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC27_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC27_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 74681 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74715 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC27_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 74725 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 74757 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 74763 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC27_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC27_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 74782 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74816 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC27_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 74826 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 74858 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 74864 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC27_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC27_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 74883 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74922 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC27_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 74932 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74985 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74992 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC27_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC27_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 75011 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75050 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC27_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 75060 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75113 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75120 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC27_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC27_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 75139 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75178 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC27_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 75188 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75241 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75248 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC27_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC27_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 75267 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75306 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC27_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 75316 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75369 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75376 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC27_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC27_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 75395 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75434 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC27_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 75444 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75497 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75504 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC27_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC27_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 75523 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75562 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC27_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 75572 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75625 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75632 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC27_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC27_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 75651 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75690 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC27_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 75700 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75753 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75760 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC27_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC27_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 75779 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75818 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC27_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 75828 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75881 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75888 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC28_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC28_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 75907 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75946 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC28_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 75956 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 75978 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75985 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC28_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC28_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 76004 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76043 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC28_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 76053 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 76075 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76082 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC28_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC28_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 76101 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76135 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC28_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 76145 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 76177 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 76183 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC28_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC28_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 76202 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76236 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC28_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 76246 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 76278 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 76284 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC28_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC28_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 76303 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76342 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC28_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 76352 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76405 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76412 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC28_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC28_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 76431 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76470 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC28_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 76480 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76533 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76540 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC28_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC28_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 76559 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76598 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC28_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 76608 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76661 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76668 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC28_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC28_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 76687 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76726 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC28_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 76736 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76789 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76796 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC28_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC28_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 76815 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76854 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC28_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 76864 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76917 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76924 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC28_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC28_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 76943 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76982 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC28_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 76992 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77045 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77052 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC28_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC28_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 77071 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77110 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC28_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 77120 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77173 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77180 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC28_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC28_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 77199 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77238 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC28_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 77248 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77301 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77308 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC29_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC29_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 77327 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77366 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC29_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 77376 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 77398 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77405 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC29_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC29_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 77424 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77463 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC29_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 77473 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 77495 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77502 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC29_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC29_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 77521 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77555 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC29_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 77565 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 77597 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 77603 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC29_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC29_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 77622 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77656 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC29_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 77666 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 77698 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 77704 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC29_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC29_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 77723 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77762 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC29_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 77772 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77825 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77832 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC29_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC29_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 77851 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77890 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC29_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 77900 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77953 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77960 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC29_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC29_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 77979 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78018 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC29_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 78028 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78081 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78088 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC29_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC29_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 78107 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78146 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC29_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 78156 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78209 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78216 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC29_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC29_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 78235 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78274 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC29_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 78284 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78337 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78344 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC29_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC29_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 78363 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78402 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC29_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 78412 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78465 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78472 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC29_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC29_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 78491 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78530 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC29_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 78540 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78593 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78600 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC29_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC29_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 78619 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78658 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC29_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 78668 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78721 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78728 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC30_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC30_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 78747 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78786 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC30_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 78796 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 78818 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78825 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC30_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC30_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 78844 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78883 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC30_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 78893 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 78915 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78922 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC30_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC30_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 78941 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78975 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC30_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 78985 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 79017 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 79023 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC30_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC30_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 79042 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79076 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC30_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 79086 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 79118 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 79124 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC30_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC30_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 79143 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79182 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC30_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 79192 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79245 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79252 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC30_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC30_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 79271 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79310 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC30_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 79320 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79373 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79380 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC30_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC30_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 79399 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79438 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC30_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 79448 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79501 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79508 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC30_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC30_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 79527 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79566 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC30_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 79576 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79629 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79636 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC30_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC30_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 79655 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79694 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC30_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 79704 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79757 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79764 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC30_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC30_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 79783 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79822 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC30_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 79832 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79885 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79892 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC30_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC30_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 79911 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79950 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC30_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 79960 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80013 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80020 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC30_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC30_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 80039 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80078 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC30_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 80088 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80141 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80148 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC31_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC31_Rc0
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 80167 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80206 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC31_Rc0;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 80216 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 80238 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80245 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC31_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC31_Rc1
#line 4744 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 80264 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4744 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80303 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC31_Rc1;

    /* semantic routine */
#line 4744 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 80313 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4744 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 80335 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80342 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC31_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC31_Rc0
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 80361 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80395 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC31_Rc0;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 80405 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 80437 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 80443 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC31_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC31_Rc1
#line 3848 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 80462 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3848 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80496 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC31_Rc1;

    /* semantic routine */
#line 3848 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 80506 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3848 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 80538 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 80544 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC31_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC31_Rc0
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 80563 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80602 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC31_Rc0;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 80612 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80665 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80672 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC31_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC31_Rc1
#line 4060 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 80691 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4060 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80730 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC31_Rc1;

    /* semantic routine */
#line 4060 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 80740 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4060 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80793 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80800 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC31_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC31_Rc0
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 80819 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80858 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC31_Rc0;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 80868 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80921 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80928 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC31_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC31_Rc1
#line 3966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 80947 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3966 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80986 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC31_Rc1;

    /* semantic routine */
#line 3966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 80996 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3966 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81049 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81056 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC31_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC31_Rc0
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 81075 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 81114 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC31_Rc0;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 81124 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81177 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81184 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC31_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC31_Rc1
#line 4248 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 81203 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4248 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 81242 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC31_Rc1;

    /* semantic routine */
#line 4248 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 81252 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4248 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81305 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81312 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC31_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC31_Rc0
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 81331 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 81370 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC31_Rc0;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 81380 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81433 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81440 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC31_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC31_Rc1
#line 4154 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 81459 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4154 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 81498 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC31_Rc1;

    /* semantic routine */
#line 4154 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 81508 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4154 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81561 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81568 "icache.c"

#undef insn
  }
}

#endif /* _ICACHE_C_*/
